#!/usr/bin/env python3
import pandas as pd
import numpy as np
import statsmodels.formula.api as smf
import statsmodels.api as sm
import os

data_dir = "../data/"
figures_dir = "../figures_new/"
kingdoms = ["Bacteria","Archaea","Eukaryota"]
twokingdoms = ["Prokaryota","Eukaryota"]

if not os.path.exists(figures_dir):
    os.makedirs(figures_dir)
        
# open the three reference files
genomes_overview_file = data_dir + "overview.txt"
genomes_euks_file = data_dir + "eukaryotes.txt"
genomes_proks_file = data_dir + "prokaryotes.txt"
    
df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
df_reference = pd.concat([df_euks, df_proks],sort=True)

taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
taxid2kingdom = {}

for t in taxid2group:
    gr = taxid2group[t]
    taxid2kingdom[t] = group2kingdom[gr]
    
taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


# load random genes
df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")


# load datasets (per species)
pfam_sets = ["full", "pfam", "domains", "linkers", "N-linkers", "M-linkers", "C-linkers"]
uniprot_sets = ["All","Shared","Unique","None"]
datasets=uniprot_sets+pfam_sets

pfam_sets2 = ["pfam", "domains", "linkers"]
uniprot_sets2 = ["Shared","Unique","None"]
datasets2=uniprot_sets2+pfam_sets2
#datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

mycotaxa = []
eurotaxa = []
for t in name2taxid:
    tax = t.split()[0]
    if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
        mycotaxa += [name2taxid[t]]
    if tax in ["Euryarchaeota"]:
        eurotaxa += [name2taxid[t]]

    
dfs = {}

aas_sorted_topidp =  ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
aas_sorted_bacfreq = ['L','A','G','V','E','I','S','R','D','T','K','P','F','N','Q','Y','M','H','W','C']


for d in datasets:
    df=pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-GC-" + d+ ".csv").dropna(axis=0)
    # Now we add a "prokayotes" kingdom
    tempdf=df[((df.kingdom == "Archaea") | (df.kingdom == "Bacteria"))]
    tempdf.kingdom="Prokaryota"
    dfs[d]=pd.concat([df, tempdf], ignore_index=True)
    # sys.exit()

df=dfs["linkers"]
df=df.dropna()
good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

props = ["freq_" + aa for aa in aas_sorted_topidp]
aa_dict = {}
for aa_col in [col for col in props]:
    lm = smf.ols( aa_col + ' ~ C(kingdom)+ GC_genomic', data=df).fit()
    table = sm.stats.anova_lm(lm, typ=2)
    aa_dict[aa_col] = table["F"]["C(kingdom)"]
    print(aa_col)
    print(table)


fv = pd.DataFrame()
fv = fv.append(aa_dict,ignore_index=True).T.rename(columns={0: "F"})
fv.sort_values(by="F",ascending=False,inplace=True)
print (fv)

