#!/usr/bin/env python3
from __future__ import print_function
import pandas as pd
import os
#import functools
#import matplotlib 
#matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import math
#from scipy.stats.stats import rankdata, square_of_sums, tiecorrect
#from scipy.stats import stats
#from scipy.stats.stats import linregress
import pickle

def fmt_scientific(x):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)
    

# define a function to plot a bar plot of the difference between AA frequencies in two datasets
# d1, d2 = the two dataset (could be the same dataset); datasets are "dataset","dataset-eggnog","dataset-pfam","dataset-pfam-full"
# k1, k2 = the kingdoms: can be either "Eukaryota" or "Bacteria"
def bar_plot_s1_vs_s2(d1,d2,k1,k2,figname=None):
    data_aa = []
    data_log = []
    data_weighted = []
    sum_weighted=0
    
    for aa in aas_sorted_topidp:
        prop = "freq_" + aa

        prop_values = {}
        # We have change the pickles
        #prop_values_1 = float(dic_res[d1][k1][prop])/float(dic_res[d1][k1]["total_residues"])
        #prop_values_2 = float(dic_res[d2][k2][prop])/float(dic_res[d2][k2]["total_residues"])
        prop_values_1 = float(dic_res[d1][k1][prop])*100
        prop_values_2 = float(dic_res[d2][k2][prop])*100

        value = prop_values_1-prop_values_2
        value_log = np.log(prop_values_1/prop_values_2)
        value_weighted = value*top_idp[aa]*100
            
        data_aa += [value]
        data_log += [value_log]
        data_weighted += [value_weighted]
        sum_weighted+=value_weighted
        
    data_weighted += [sum_weighted]
    xlabels=aas_sorted_topidp.copy()
    xlabels += ["sum"]

    title = dic_labels[k1] + " " + d1 +  " vs "  + dic_labels[k2]+ " " + d2
    
    clr=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red"]
    clr_green=["green","green","green","green","green","green","green","green","green","green","green","green","green","green","green","green","green","green","green","green"]
    clr_weighted=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue"]

    # Raw frequency plot
    fig, ax = plt.subplots(figsize=(6.,3))
    ax.bar(range(len(data_aa)),data_aa, color=clr)
    ax.set_xticks([0.0 + x for x in range(len(aas_sorted_topidp))])
    ax.set_xticklabels(xlabels)
    ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
    ax.set_title(title)
    ax.yaxis.grid()
    ax.xaxis.grid()
    if figname != None:
        filename=figname+"frequency.eps"
        #print(filename)
        fig.savefig(filename, bbox_inches = "tight")
    else:
        plt.show()
    plt.close()


    # Weighted frequency plot
    fig, ax = plt.subplots(figsize=(6.,3))
    ax.bar(range(len(data_weighted)),data_weighted, color=clr_weighted)
    ax.set_xticks([0.0 + x for x in range(len(xlabels))])
    ax.set_xticklabels(xlabels)
    ax.set_ylabel( "TOPIDP ("+k1+" - " + k2+ ")")
    ax.set_title(title)
    ax.yaxis.grid()
    ax.xaxis.grid()
    if figname != None:
        filename=figname+"weighted.eps"
        #print(filename)
        fig.savefig(filename, bbox_inches = "tight")
    else:
        plt.show()
    plt.close()



    # Log frequency plot
    fig, ax = plt.subplots(figsize=(6.,3))
    ax.bar(range(len(data_log)),data_log, color=clr)
    ax.set_xticks([0.0 + x for x in range(len(aas_sorted_topidp))])
    ax.set_xticklabels(xlabels)
    ax.set_ylabel("log( freq(" + k1+ ") / freq(" + k2+ ") )")
    ax.set_title(title)
    ax.yaxis.grid()
    ax.xaxis.grid()
    if figname != None:
        filename=figname+"logodds.eps"
        #print(filename)
        fig.savefig(filename, bbox_inches = "tight")
    else:
        plt.show()
    plt.close()

    bar_width=0.45
    index = np.arange(20)

    # Combined frequency plot
    fig, ax = plt.subplots(figsize=(6.,3))
    ax.bar(range(len(data_weighted)),data_weighted,bar_width, color=clr_weighted)
    ax.bar(index+bar_width,data_log,bar_width, color="green",alpha=0.5)
    ax.set_xticks([bar_width/2 + x for x in range(len(xlabels))])
    ax.set_xticklabels(xlabels)
    ax.set_ylabel( "Combined ("+k1+" - " + k2+ ")")
    ax.set_title(title)
    ax.yaxis.grid()
    ax.xaxis.grid()
    if figname != None:
        filename=figname+"combo.eps"
        #print(filename)
        fig.savefig(filename, bbox_inches = "tight")
    else:
        plt.show()
    plt.close()


#************************************************************************





font = {'size'   : 22}

aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
aas =               ['A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y']
aas_sorted =        ['A','R','N','D','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y']

# Top-IDP scale
top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
    'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
    'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
    'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
    'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

top_idp_noser = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
    'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
    'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
    'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
    'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
    
top_idp_noserlyspro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
    'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
    'K' : 0., 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
    'P' : 0.0, 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
    'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
    
dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",   "top-idp":"<TOP-IDP>", "topidp-S":"<TOP-IDP-noS>","topidp-SKP":"<TOP-IDP-noSKP>", "topidp-freq":"<TOP-IDP-FREQ>",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length_translation":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(AA)','iupred_cutoff':'IUpred cutoff (AA)',
                'idp':'<TOP-IDP>','tm':'TM (AA)','transmembrane':'TM(AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(AA)','iupred_cutoff':'IUpred cutoff(AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':'TYR',
                'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                'freq_T': 'THR', 'freq_W': 'TRP', 'freq_V': 'VAL',
                'freq_Y': 'TYR',
                'freq_A-shift': 'ALA', 'freq_C-shift': 'CYS', 'freq_E-shift': 'GLU',
                'freq_D-shift': 'ASP', 'freq_G-shift': 'GLY', 'freq_F-shift': 'PHE',
                'freq_I-shift': 'ILE', 'freq_H-shift': 'HIS', 'freq_K-shift': 'LYS',
                'freq_M-shift': 'MET', 'freq_L-shift': 'LEU', 'freq_N-shift': 'ASN',
                'freq_Q-shift': 'GLN', 'freq_P-shift': 'PRO', 'freq_S-shift':'SER', 'freq_R-shift': 'ARG',
                'freq_T-shift': 'THR', 'freq_W-shift': 'TRP', 'freq_V-shift': 'VAL',
                'freq_Y-shift': 'TYR',
                'freq_A-fifty': 'ALA', 'freq_C-fifty': 'CYS', 'freq_E-fifty': 'GLU',
                'freq_D-fifty': 'ASP', 'freq_G-fifty': 'GLY', 'freq_F-fifty': 'PHE',
                'freq_I-fifty': 'ILE', 'freq_H-fifty': 'HIS', 'freq_K-fifty': 'LYS',
                'freq_M-fifty': 'MET', 'freq_L-fifty': 'LEU', 'freq_N-fifty': 'ASN',
                'freq_Q-fifty': 'GLN', 'freq_P-fifty': 'PRO', 'freq_S-fifty':'SER', 'freq_R-fifty': 'ARG',
                'freq_T-fifty': 'THR', 'freq_W-fifty': 'TRP', 'freq_V-fifty': 'VAL',
                'freq_Y-fifty': 'TYR',
                  'scampi':'TM  (AA)', 'slider':'Slider',
                  'disembl_coils':'DisEMBL Coils (AA)',
                      'disembl_rem465':"DisEMBL REM465 (AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (AA)",
                      "globpipe":"GlobPlot (AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea',
                "full":"Shared Proteins", "pfam":"Shared domains", "domains":"Specific domains", 
                "linkers":"Linker regions","space":"",
                "N-linkers":"N-terminal Linker regions",
                "M-linkers":"Central Linker regions",
                "C-linkers":"C-terminal Linker regions",
                  "All":"All proteins","Unique":"Specific Proteins","Shared":"Shared Proteins","None":"No Domain proteins"
                     ,"numdiso_long":"Number of disorder residues (long)"
                     ,"numdiso_short":"Number of disorder residue (short)"
                     ,"numseg":"Low complexity residues"
                  }                  
ylabels = {"GC":"Fraction","GC_genomic":"Fraction", 'other':'Intermediate',
        'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity>", "top-idp":"<TOP-IDP>","topidp-S":"<TOP-IDP-noS>","topidp-SKP":"<TOP-IDP-noSKP>","topidp-freq":"<TOP-IDP-FREQ>",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'fraction','taxon_gc':'fraction','taxon_gc_r':'fraction',
    'random':'fraction','iupred_long':'Fraction of AA ','length_aa':'<Length>','gc':'fraction',
    'iupred_short':'Fration of residues','iupred_cutoff':'Fraction of residues ', 'idp':'<TOP-IDP>','tm':'Fraction of residues ',"volatility":"<Volatility>", 'iupred_short':'Fraction of residues ','iupred_cutoff':'Fraction of residues','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity>','gc':'Fraction','aa_corr':'Correlation', 'aa_corr_dot':'correlation','aa_corr_r':'Correlation', 'A':'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'Fraction of residues ', 'freq_C':
    'Fraction of residues ', 'freq_E': 'Fraction of residues ', 'freq_D': 'Fraction of residues', 'freq_G': 'Fraction of residues ',
    'freq_F': 'Fraction of residues ', 'freq_I': 'Fraction of residues ', 'freq_H': 'Fraction of residues ', 'freq_K':
    'Fraction of residues ', 'freq_M': 'Fraction of residues ', 'freq_L': 'Fraction of residues ', 'freq_N': 'Fraction of residues ',
    'freq_Q': 'Fraction of residues ', 'freq_P': 'Fraction of residues ', 'freq_S': 'Fraction of residues ', 'freq_R':
    'Fraction of residues ', 'freq_T': 'Fraction of residues ', 'freq_W': 'Fraction of residues ', 'freq_V': 'Fraction of residues ',
    'freq_Y': 'Fraction of residues ', 'scampi':'TM (AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (AA)','disembl_rem465':"DisEMBL REM465 (AA)",
    'disembl_hotloops':"DisEMBL Hotloops (AA)", "globpipe":"GlobPlot (AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea'
               ,"numdiso_long":"Number of disorder residues (long)"
                     ,"numdiso_short":"Number of disorder residues (short)"
                     ,"numseg":"Low complexity residues"
               }
        

            
dic_boxplot_limits = {"iupred_long":[0,50], "iupred_short":[0,50], 
                      "top-idp":[0,0.15],"topidp-S":[0,0.15],"topidp-SKP":[-0.05,0.05],"topidp-freq":[0,0.15],'length_translation':[0,550],
                      u'hessa':[0.85,1.2], 'seg':[0,25],"scampi":[0.02,0.1],
                    "freq_S":[0.03,0.1],"freq_P":[0.01,0.08], 
                      "freq_I":[0.02,0.11],"freq_T":[0.03,0.08],
                              "freq_A":[0.00,0.18],
                              "freq_C":[0.0,0.06],
                              "freq_D":[0.02,0.1],
                              "freq_E":[0.01,0.12],
                              "freq_F":[0.00,0.1],
                              "freq_G":[0.03,0.15],
                              "freq_H":[0.00,0.06],
                              "freq_K":[0.00,0.12],
                              "freq_L":[0.05,0.15],
                              "freq_M":[0.01,0.05],
                              "freq_N":[0.00,0.1],
                              "freq_Q":[0.01,0.1],
                              "freq_R":[0.01,0.1],
                              "freq_V":[0.03,0.12],
                              "freq_W":[0.0,0.04],
                              "freq_Y":[0.00,0.07],
                              }

dic_colors= {"Eukaryota":'#00B060', "Bacteria":'#E20048', "Archaea":'#0014b0'}

allsets= ["full", "pfam", "domains", "linkers","N-linkers","M-linkers","C-linkers","All","None","Unique","Shared"]
allsets2= ["None","Unique","full", "pfam", "domains", "linkers","N-linkers","M-linkers","C-linkers","All"]


#datasets = ["All","Unique","None", "Shared","pfam", "domains", "linkers",]

protsets = ["All","Shared","Unique","None"]
domsets = ["pfam", "domains", "linkers"]

#datasets = protsets+domsets

dic_res = {}
for ty in allsets:
    out_dic_fname = "props_dic_GC_"+ty+".pickle"
    #print (ty)
    with open(out_dic_fname, 'rb') as handle:
        dic_res.update(pickle.load(handle))

#print (dic_res)
#data_dir="../data/"
#for d in datasets:
#    dfs[d]=pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-GC-" + d+ ".csv").dropna(axis=0)


figures_dir = "../figures/"

if not os.path.exists(figures_dir):
    os.makedirs(figures_dir)
        
    

#### plot the barplots of the differences in AA frequencies between different sets
for k1 in ["Bacteria", "Eukaryota","Archaea"]:
    for k2 in ["Bacteria", "Eukaryota","Archaea"]:
        for d1 in domsets:
            for d2 in domsets:
                if ( ((k1 == k2) and (d1 != d2)) or ((k1 != k2) and (d1 == d2)) ):
                    figname = figures_dir + "aa_freq_change_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + "_"
                    #print(k1,k2,d1,d2)
                    bar_plot_s1_vs_s2(d1,d2,k1,k2,figname)


#  Get top-idp-S data...
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in allsets:
        score =0.
        scoreS =0.
        scoreSKP =0.
        for aa in aas_sorted_topidp: 
            score+=top_idp[aa]*dic_res[ty][kingdom]["freq_"+aa]
            scoreS+=top_idp_noser[aa]*dic_res[ty][kingdom]["freq_"+aa]
            scoreSKP+=top_idp_noserlyspro[aa]*dic_res[ty][kingdom]["freq_"+aa]
        dic_res[ty][kingdom]['topidp-SKP']=scoreSKP
        dic_res[ty][kingdom]['topidp-S']=scoreS
        dic_res[ty][kingdom]['topidp-freq']=score
        #print (kingdom,ty,dic_res[ty][kingdom]["top-idp"],score,scoreS,scoreSKP)
        dic_res[ty][kingdom]["numdiso_long"]=dic_res[ty][kingdom]["iupred_long"]*dic_res[ty][kingdom]["length_translation"]
        dic_res[ty][kingdom]["numdiso_short"]=dic_res[ty][kingdom]["iupred_short"]*dic_res[ty][kingdom]["length_translation"]
        dic_res[ty][kingdom]["numseg"]=dic_res[ty][kingdom]["seg"]*dic_res[ty][kingdom]["length_translation"]
        

#~ #### barplots of average properties, euk vs bac
props = ["iupred_long","iupred_short", "seg","top-idp","hessa","length_translation"]
#props += ['topidp-freq']
props += ['numdiso_long','numdiso_short','numseg']
props += ["freq_" + aa for aa in aas_sorted_topidp]
aaprops = ["freq_" + aa for aa in aas_sorted_topidp]
props += ["ss_alpha", "ss_beta", "ss_coil", "ss_turn"]

#~ diclen = {"Eukaryota":4932465,"Bacteria": 8726710}
#props = ["length_translation"]

extraset= ["Unique","None", "Shared","space","pfam", "domains", "linkers"]

#print (dic_res)
for prop in props:
    
    fig, ax = plt.subplots()
    
    #print(prop)
    offset_x = 0

    alldata=[]
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:

        data = []
        
        for ty in extraset:
            if ty=="space":
                data += [0]
            else:
                data += [dic_res[ty][kingdom][prop]]
        #print ("stat:",prop,kingdom,data)
        alldata+=data
        ax.bar([x+offset_x for x in range(len(data))], data, color=dic_colors[kingdom],width=0.3,label=dic_labels[kingdom])
        
        offset_x += 0.3
        
    ax.set_xticks([x+0.2 for x in range(len(data))])
    ax.set_xticklabels([dic_labels[x] for x in extraset], rotation=15 )
    
    #if prop in dic_boxplot_limits:
    #    ax.set_ylim(dic_boxplot_limits[prop])
    
    ax.yaxis.grid(True)
    ax.set_title(dic_labels[prop])
    ax.set_ylabel(ylabels[prop])
    ax.legend(loc="best")
    
    figname = figures_dir + "bar_plot_"+prop
    np.savetxt(figname+".csv", alldata, delimiter=' , ', fmt='%8.2f')
    fig.savefig(figname+".eps")
    plt.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)
    #plt.show()
    plt.close()
    

    
    fig, ax = plt.subplots()
    #print(prop)
    offset_x = 0
    alldata=[]
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:

        data = []
        
        for ty in domsets:
            data += [dic_res[ty][kingdom][prop]]
        alldata+=data
        ax.bar([x+offset_x for x in range(len(data))], data, color=dic_colors[kingdom],width=0.3,label=dic_labels[kingdom])
        
        offset_x += 0.3
        
    ax.set_xticks([x+0.2 for x in range(len(data))])
    ax.set_xticklabels([dic_labels[x] for x in domsets], rotation=15 )
    
    #if prop in dic_boxplot_limits:
    #    ax.set_ylim(dic_boxplot_limits[prop])
    
    ax.yaxis.grid(True)
    ax.set_title(dic_labels[prop])
    ax.set_ylabel(ylabels[prop])
    ax.legend(loc="best")
    
    figname = figures_dir + "bar_plot_dom_"+prop
    np.savetxt(figname+".csv", alldata, delimiter=' , ', fmt='%8.2f')
    fig.savefig(figname+".eps")
    plt.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)
    #plt.show()
    plt.close()

    # Now only
    fig, ax = plt.subplots()
    
    offset_x = 0
    alldata=[]
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:

        data = []
        
        for ty in protsets:
            data += [dic_res[ty][kingdom][prop]]
        alldata+=data
        ax.bar([x+offset_x for x in range(len(data))], data, color=dic_colors[kingdom],width=0.3,label=dic_labels[kingdom])
        
        offset_x += 0.3
        
    ax.set_xticks([x+0.2 for x in range(len(data))])
    ax.set_xticklabels([dic_labels[x] for x in protsets], rotation=15 )
    
    #if prop in dic_boxplot_limits:
    #    ax.set_ylim(dic_boxplot_limits[prop])
    
    ax.yaxis.grid(True)
    ax.set_title(dic_labels[prop])
    ax.set_ylabel(ylabels[prop])
    ax.legend(loc="best")
    
    figname = figures_dir + "bar_plot_prot_"+prop
    np.savetxt(figname+".csv", alldata, delimiter=' , ', fmt='%8.2f')
    fig.savefig(figname+".eps")
    plt.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)
    #plt.show()
    plt.close()
    
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:

    print ("\\begin{table}[ht]{\\tiny\\begin{tabular}{p{2cm}|p{1cm}p{1cm}p{1cm}|p{1cm}p{1cm}p{1cm}|p{1cm}p{1cm}p{1cm}p{1cm}|p{1cm}p{1cm}p{1cm}p{1cm}}")

    for prop in props:
        head = kingdom
        for ty in allsets2:
            head += " & " + dic_labels[ty]
    
    print (head, "\\\\")
    print ({"\\hline"})

    # Just to print it into a latex table
    for prop in props:
        string=dic_labels[prop]
        for ty in allsets2:
            value = float(dic_res[ty][kingdom][prop])
            string += " & " + str(round(value,4)) 
        string += "\\\\ "
        print (string)
    print ("\\end{tabular}}")
    print ("\\caption{Summary of features for different set of proteins and protein regions in " + kingdom +" }")
    print ("\label{tab:summary-"+kingdom+"}")
    print ("\\end{table}")


# A second table
print ("\\begin{table*}[ht]{\\small\\begin{tabular}{p{1cm}|p{1.3cm}p{1.3cm}p{1.3cm}|p{1.3cm}p{1.3cm}p{1.3cm}|p{1.3cm}p{1.3cm}p{1.3cm}}")
head = '  '
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    head += " & & " + kingdom +  " &  "
head += "\\\\\n"
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in domsets:
        head += " & " + dic_labels[ty]
print (head, "\\\\ \n \\hline \n")

# Just to print it into a latex table
for prop in aaprops:
    string=dic_labels[prop]
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
        for ty in domsets:
            value = float(dic_res[ty][kingdom][prop])*100
            string += " & " + str(round(value,1)) + "\\% " 
    string += "\\\\ "
    print (string)
print ("\\end{tabular}}")
print ("\\caption{Amino acid frequencies for each protein region in all kingdoms.  }")
print ("\label{tab:frequency}")
print ("\\end{table*}")


newprops = ["freq_" + aa + "-fifty" for aa in aas_sorted_topidp]
# A third  table (fifty)
print ("\\begin{table*}[ht]{\\small\\begin{tabular}{p{1cm}|p{1.3cm}p{1.3cm}p{1.3cm}|p{1.3cm}p{1.3cm}p{1.3cm}|p{1.3cm}p{1.3cm}p{1.3cm}}")
head = '  '
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    head += " & & " + kingdom +  " &  "
head += "\\\\\n"
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in domsets:
        head += " & " + dic_labels[ty]
print (head, "\\\\ \n \\hline \n")

# Just to print it into a latex table
for prop in newprops:
    string=dic_labels[prop]
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
        for ty in domsets:
            value = float(dic_res[ty][kingdom][prop])*100
            string += " & " + str(round(value,1)) + "\\% " 
    string += "\\\\ "
    print (string)
print ("\\end{tabular}}")
print ("\\caption{Amino acid frequencies for each protein region and kingdom assuming 50\\% GC.  }")
print ("\\end{table*}")
    
#######  HEATMAP
#props = ["freq_" + aa for aa in aas_sorted_topidp]
data = []
y_labels = []
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in domsets:
        if ty != "full":
            d = []
            for prop in aaprops:
                value = float(dic_res[ty][kingdom][prop])
                d += [value]
            y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
            data += [d]
import seaborn as sns
sns.set()
sns.set(font_scale=1.0)
np.savetxt("freq.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')
sns.clustermap(data, col_cluster=False,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')
plt.savefig(figures_dir + "aa_freq_heatmap.eps",bbox_inches="tight")

#######  DomHEATMAP
#props = ["freq_" + aa for aa in aas_sorted_topidp]
data = []
y_labels = []
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in domsets:
        if ty != "Shared":
            d = []
            for prop in aaprops:
                value = float(dic_res[ty][kingdom][prop])
                d += [value]
            y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
            data += [d]
import seaborn as sns
sns.set()
sns.set(font_scale=1.0)
np.savetxt("freq-dom.csv", data, delimiter=' & ', fmt='%2.2f', newline=' \\\\\n')
sns.clustermap(data, col_cluster=True,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')
plt.savefig(figures_dir + "aa_freq_dom_heatmap.eps",bbox_inches="tight")

#######  HEATMAP
#props = ["freq_" + aa for aa in aas_sorted_topidp]
data = []
y_labels = []
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in domsets:
        if ty != "full":
            d = []
            for prop in newprops:
                value = float(dic_res[ty][kingdom][prop])
                d += [value]
            y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
            data += [d]
import seaborn as sns
sns.set()
sns.set(font_scale=1.0)
np.savetxt("freq-fifty.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')
sns.clustermap(data, col_cluster=True,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')
plt.savefig(figures_dir + "aa_fifty_heatmap.eps",bbox_inches="tight")

#######  DomHEATMAP
#props = ["freq_" + aa for aa in aas_sorted_topidp]
data = []
y_labels = []
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for ty in domsets:
        if ty != "Shared":
            d = []
            for prop in newprops:
                value = float(dic_res[ty][kingdom][prop])
                d += [value]
            y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
            data += [d]
import seaborn as sns
sns.set()
sns.set(font_scale=1.0)
np.savetxt("freq-dom-fifty.csv", data, delimiter=' & ', fmt='%2.2f', newline=' \\\\\n')
sns.clustermap(data, col_cluster=False,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')
plt.savefig(figures_dir + "aa_fifty_dom_heatmap.eps",bbox_inches="tight")
