#!/usr/bin/env python3
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import string
import math
import os.path
from Bio import motifs
import sys
import csv
from random import *
import numpy
from scipy.stats.stats import rankdata, tiecorrect
from scipy.stats import stats
#from statsmodels.formula.api import ols
from scipy.stats.stats import linregress
from scipy.stats import ttest_ind
from scipy.stats import mannwhitneyu
from scipy.spatial import distance
from scipy.stats import norm

def count_subst(a,b):
    for i in len(a):
        i=j
    return i

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

def parse_annotation(filename):

    tax_id = int(filename.split("_")[1].split(".")[0])
    print (tax_id)

    df = pd.read_csv(filename)
    n_proteins = len(df.query_id)

    df_mean = df.mean()
    
    columns = ["length", "top-idp", "iupred_long", "iupred_short","seg","ss_alpha", "ss_beta", "ss_coil", "ss_turn","hessa"]
    columns += ["freq_" + aa for aa in aas]
    columns += ["pfamA_acc"]
    
    ret_dic = {}    
    ret_dic["taxon_id"] = tax_id
    ret_dic["phylum"] = taxid2phylum.get(tax_id,pd.np.nan)
    ret_dic["kingdom"] = taxid2kingdom.get(tax_id,pd.np.nan)
    ret_dic["GC"] = taxid2gc.get(tax_id,pd.np.nan)
    ret_dic["count_protein"] = n_proteins

    for c in columns:
        ret_dic[c] = df_mean[c]
    
    #gcs = df_reference.loc[df_reference["TaxID"] == tax_id]["GC%"].astype(float)
    #ret_dic["GC"] = pd.np.mean(list(gcs))

    return ret_dic

def twoSampZ(X1, X2, mudiff, sd1, sd2, n1, n2):
        pooledSE = sqrt(sd1**2/n1 + sd2**2/n2)
        z = ((X1 - X2) - mudiff)/pooledSE
        pval = 2*(1 - norm.cdf(abs(z)))
        return round(z, 3), round(pval, 4)
                
if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
    aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
                   'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
                   'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}
    ssindex = {'H':0, 'I':0, 'G':0, 'E':1, 'B':1, 'S':2, 'T':2, 'C':2}
    STDOUT = csv.writer(sys.stdout)
    STDOUT.writerow( ( "pfam", "aa","length","bac_freq","arch_freq","euk_freq","numbac","numarch","numeuk","P-mann","T-mann","Z-Ttest","P-Ttest","mean-bac","mean-arch","mean-euk","sd-bac","sd-arch","sd-euk","Z"))
    data_dir = "../data/"
    results_dir = "../data/"
    input_dir = "../data/pfam/"
    output_dir = "../data/"
    
    # This takes too long
    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)

    # We allow max 20 (10 currently) GO terms per Pfam family
    maxgolist=20
    maxpfamlist=400
    maxpfam=20000
    maxgo=9999999
    pfam2go=numpy.zeros((maxpfam,maxgolist))
    go2pfam=numpy.zeros((maxgo,maxpfamlist))
    pfamlist={}
    golist={}
    fasta_list = []
    pfam_list = []
    for f in os.listdir(input_dir):
        if f.endswith("_bac_medGC.fa"):
            pfam=re.sub(r'_bac_medGC.fa','',f)
            bacfile=pfam+"_bac_medGC.fa"
            archfile=pfam+"_arch_medGC.fa"
            eukfile=pfam+"_euk_medGC.fa"
            if (os.path.isfile(input_dir+bacfile) and  os.path.isfile(input_dir+archfile) and  os.path.isfile(input_dir+eukfile)):
                if (os.path.getsize(input_dir+bacfile)>0 and  os.path.getsize(input_dir+archfile)>0 and  os.path.getsize(input_dir+eukfile)>0):
                    arch_freq=numpy.zeros(20)
                    bac_freq=numpy.zeros(20)
                    euk_freq=numpy.zeros(20)
                    arch= AlignIO.read(input_dir+pfam+"_arch_medGC.fa", "fasta")
                    bac= AlignIO.read(input_dir+pfam+"_bac_medGC.fa", "fasta")
                    euk = AlignIO.read(input_dir+pfam+"_euk_medGC.fa", "fasta")
                    length=len(bac[0].seq)
                    numbac=len(bac)
                    numarch=len(arch)
                    numeuk=len(euk)
                    bacseq=numpy.zeros((numbac,20))
                    archseq=numpy.zeros((numarch,20))
                    eukseq=numpy.zeros((numeuk,20))
                    #print ("LENGTH",length,len(bac),len(euk))
                    for j in range(0,length):
                        for i in range(0,len(bac)):
                            seq=bac[i].seq[j].upper()
                            if (re.match('[ACDEFGHIKLMNPQRSTVWY]',seq)):
                                bac_freq[aaindex[seq]]+=1
                                bacseq[i,aaindex[seq]]+=1
                    for j in range(0,length):
                        for i in range(0,len(arch)):
                            seq=arch[i].seq[j].upper()
                            if (re.match('[ACDEFGHIKLMNPQRSTVWY]',seq)):
                                arch_freq[aaindex[seq]]+=1
                                archseq[i,aaindex[seq]]+=1
                    for j in range(0,length):
                        for i in range(0,len(euk)):
                            seq=euk[i].seq[j].upper()
                            if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                                euk_freq[aaindex[seq]]+=1
                                eukseq[i,aaindex[seq]]+=1

                    bacseq=bacseq/bacseq.sum(axis=1, keepdims=True)
                    archseq=archseq/archseq.sum(axis=1, keepdims=True)
                    eukseq=eukseq/eukseq.sum(axis=1, keepdims=True)
                    for i in range(20):
                        a=archseq[:, [i]]
                        b=bacseq[:, [i]]
                        e=eukseq[:, [i]]
                        #print ("Bac",b)
                        #print ("Euk",e)
                        (A, B) = ttest_ind(b, e,equal_var=False)
                        #print ("T-test",T,P,numpy.mean(b),numpy.mean(e),numpy.sqrt(numpy.var(b)),numpy.sqrt(numpy.var(b)))
                        try: 
                            (T,P) = mannwhitneyu(b,e,alternative='two-sided')
                        except:
                            T=0.
                            P=9999.
                        #print ("mann",T,P)
                        mean_bac=numpy.mean(b)
                        mean_arch=numpy.mean(a)
                        mean_euk=numpy.mean(e)
                        sd_bac=numpy.sqrt(numpy.var(b))
                        sd_arch=numpy.sqrt(numpy.var(a))
                        sd_euk=numpy.sqrt(numpy.var(e))
                        Z=(mean_euk-mean_bac)/math.sqrt(sd_euk*sd_euk+sd_bac*sd_bac+1.e-20)
                        STDOUT.writerow( ( pfam, aas[i],length,bac_freq[i]/sum(bac_freq),
                                           arch_freq[i]/sum(arch_freq),euk_freq[i]/sum(euk_freq),
                                           numbac,numarch,numeuk,P,T,A[0],B[0],
                                           mean_bac,mean_arch,mean_euk,
                                           sd_bac,sd_arch,sd_euk,Z))

