#!/usr/bin/env python3
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
from Bio.PDB import *
import os.path
from Bio import motifs
import sys
from random import *
import numpy

class PDBSelect(Select):
    def accept_residue(self, residue):
        if residue.get_name()=='GLY':
            return True
        else:
            return False

def count_subst(a,b):
    for i in len(a):
        i=j
    return i

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

def parse_annotation(filename):

    tax_id = int(filename.split("_")[1].split(".")[0])
    print (tax_id)

    df = pd.read_csv(filename)
    n_proteins = len(df.query_id)

    df_mean = df.mean()
    
    columns = ["length", "top-idp", "iupred_long", "iupred_short","seg","ss_alpha", "ss_beta", "ss_coil", "ss_turn","hessa"]
    columns += ["freq_" + aa for aa in aas]
    columns += ["pfamA_acc"]
    
    ret_dic = {}    
    ret_dic["taxon_id"] = tax_id
    ret_dic["phylum"] = taxid2phylum.get(tax_id,pd.np.nan)
    ret_dic["kingdom"] = taxid2kingdom.get(tax_id,pd.np.nan)
    ret_dic["GC"] = taxid2gc.get(tax_id,pd.np.nan)
    ret_dic["count_protein"] = n_proteins

    for c in columns:
        ret_dic[c] = df_mean[c]
    
    #gcs = df_reference.loc[df_reference["TaxID"] == tax_id]["GC%"].astype(float)
    #ret_dic["GC"] = pd.np.mean(list(gcs))

    return ret_dic

if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
    aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
                   'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
                   'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}
    ssindex = {'H':0, 'I':0, 'G':0, 'E':1, 'B':1, 'S':2, 'T':2, 'C':2}

    data_dir = "../data/"
    pdb_dir = "../data/"
    results_dir = "../data/"
    input_dir = "../data/pfam/"
    output_dir = "../data/"
    cifparser = MMCIFParser()
    pdbparser = PDBParser()
    
    # This takes too long
    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)

    pfam_list = []
    for f in os.listdir(input_dir):
        #if f.endswith("222-full.ss"):
        if f.endswith(".ss"):
            basename=re.sub(r'-full.ss','',f)
            bacfile=basename+"_bac_medGC.fa"
            archfile=basename+"_arch_medGC.fa"
            eukfile=basename+"_euk_medGC.fa"
            if (os.path.isfile(input_dir+bacfile) and  os.path.isfile(input_dir+eukfile) and  os.path.isfile(input_dir+archfile)):
                if (os.path.getsize(input_dir+bacfile)>0 and  os.path.getsize(input_dir+archfile)>0 and  os.path.getsize(input_dir+eukfile)>0):
                    pfam_list += [basename]
    #for f in sys.argv[1:]:
    #    file_list += [f]
    sumbac_helix=numpy.zeros(20)
    sumbac_sheet=numpy.zeros(20)
    sumbac_loop=numpy.zeros(20)
    sumbac_diso=numpy.zeros(20)
    sumarch_helix=numpy.zeros(20)
    sumarch_sheet=numpy.zeros(20)
    sumarch_loop=numpy.zeros(20)
    sumarch_diso=numpy.zeros(20)
    sumeuk_helix=numpy.zeros(20)
    sumeuk_sheet=numpy.zeros(20)
    sumeuk_loop=numpy.zeros(20)
    sumeuk_diso=numpy.zeros(20)
    #print (pfam_list)
    numdiso_euk=0
    numhelix_euk=0
    numsheet_euk=0
    numloop_euk=0
    numdiso_bac=0
    numhelix_bac=0
    numsheet_bac=0
    numloop_bac=0
    numdiso_arch=0
    numhelix_arch=0
    numsheet_arch=0
    numloop_arch=0
    for pfam in pfam_list:
        arch= AlignIO.read(input_dir+pfam+"_arch_medGC.fa", "fasta")
        bac= AlignIO.read(input_dir+pfam+"_bac_medGC.fa", "fasta")
        euk = AlignIO.read(input_dir+pfam+"_euk_medGC.fa", "fasta")
        ss = AlignIO.read(input_dir+pfam+"-full.ss", "fasta")
        

        length=len(bac[0].seq)
        numarch=len(arch)
        numbac=len(bac)
        numeuk=len(euk)
        numss=len(ss)
        arch_freq=numpy.zeros((length,20))
        bac_freq=numpy.zeros((length,20))
        euk_freq=numpy.zeros((length,20))
        ss_freq=numpy.zeros((length,3))
        #print ("LENGTH",length,len(bac),len(euk))
        for i in range(0,len(bac)):
            for j in range(0,length):
                seq=bac[i].seq[j].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                    bac_freq[j][aaindex[seq]]+=1
        for i in range(0,len(arch)):
            for j in range(0,length):
                seq=arch[i].seq[j].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                    arch_freq[j][aaindex[seq]]+=1
        for i in range(0,len(euk)):
            for j in range(0,length):
                seq=euk[i].seq[j].upper()
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', seq)):
                    euk_freq[j][aaindex[seq]]+=1
                    #print ("TEST",i,j,euk[i].seq[j],aaindex[euk[i].seq[j]],euk_freq[j][aaindex[euk[i].seq[j]]])
        for i in range(0,len(ss)):
            for j in range(0,length):
                if (re.match('[GHIEBTSC]', ss[i].seq[j])):
                    ss_freq[j][ssindex[ss[i].seq[j]]]+=1
        for j in range(0,length):
            sumbac=0.
            sumarch=0.
            sumeuk=0.
            sumss=ss_freq[j][0]+ss_freq[j][1]+ss_freq[j][2]+1.e-20
            #print ("ITER", j,sumss,ss_freq[j][0],ss_freq[j][1],ss_freq[j][2])
            if ((ss_freq[j][0]/sumss > 0.5)):
                for i in range(20):
                    sumeuk+=euk_freq[j][i]
                    sumbac+=bac_freq[j][i]
                    sumarch+=arch_freq[j][i]
                if sumeuk>0:
                    numhelix_euk+=1
                    for i in range(20):
                        sumeuk_helix[i]+=euk_freq[j][i]/sumeuk
                if sumbac>0:
                    numhelix_bac+=1
                    for i in range(20):
                        sumbac_helix[i]+=bac_freq[j][i]/sumbac
                if sumarch>0:
                    numhelix_arch+=1
                    for i in range(20):
                        sumarch_helix[i]+=arch_freq[j][i]/sumarch
            elif (ss_freq[j][1]/sumss > 0.5):
                for i in range(20):
                    sumeuk+=euk_freq[j][i]
                    sumbac+=bac_freq[j][i]
                    sumarch+=arch_freq[j][i]
                if sumeuk>0:
                    numsheet_euk+=1
                    for i in range(20):
                        sumeuk_sheet[i]+=euk_freq[j][i]/sumeuk
                if sumbac>0:
                    numsheet_bac+=1
                    for i in range(20):
                        sumbac_sheet[i]+=bac_freq[j][i]/sumbac
                if sumarch>0:
                    numsheet_arch+=1
                    for i in range(20):
                        sumarch_sheet[i]+=arch_freq[j][i]/sumarch
            elif (ss_freq[j][2]/sumss > 0.5):                
                for i in range(20):
                    sumeuk+=euk_freq[j][i]
                    sumbac+=bac_freq[j][i]
                    sumarch+=arch_freq[j][i]
                if sumeuk>0:
                    numloop_euk+=1
                    for i in range(20):
                        sumeuk_loop[i]+=euk_freq[j][i]/sumeuk
                if sumbac>0:
                    numloop_bac+=1
                    for i in range(20):
                        sumbac_loop[i]+=bac_freq[j][i]/sumbac
                if sumarch>0:
                    numloop_arch+=1
                    for i in range(20):
                        sumarch_loop[i]+=arch_freq[j][i]/sumarch
            else:
                for i in range(20):
                    sumeuk+=euk_freq[j][i]
                    sumbac+=bac_freq[j][i]
                    sumarch+=arch_freq[j][i]
                if sumeuk>0:
                    numdiso_euk+=1
                    for i in range(20):
                        sumeuk_diso[i]+=euk_freq[j][i]/sumeuk
                if sumbac>0:
                    numdiso_bac+=1
                    for i in range(20):
                        sumbac_diso[i]+=bac_freq[j][i]/sumbac
                if sumarch>0:
                    numdiso_arch+=1
                    for i in range(20):
                        sumarch_diso[i]+=arch_freq[j][i]/sumarch

    num=len(pfam_list)
    
    #print ("TEST",numhelix_bac,sumbac_helix.sum())
    print ("num","AA","bac-helix","arch-helix","euk-helix","bac-sheet","arch-sheet","euk-sheet",
           "bac-loop","arch-loop","euk-loop","bac-diso","arch-diso","euk-diso")

    for i in range(20):
        print (i,aas[i],
               round(100*sumbac_helix[i]/numhelix_bac,1),round(100*sumarch_helix[i]/numhelix_arch,1),round(100*sumeuk_helix[i]/numhelix_euk,1),
               round(100*sumbac_sheet[i]/numsheet_bac,1),round(100*sumarch_sheet[i]/numsheet_arch,1),round(100*sumeuk_sheet[i]/numsheet_euk,1),
               round(100*sumbac_loop[i]/numloop_bac,1),round(100*sumarch_loop[i]/numloop_arch,1),round(100*sumeuk_loop[i]/numloop_euk,1),
               round(100*sumbac_diso[i]/numdiso_bac,1),round(100*sumarch_diso[i]/numdiso_arch,1),round(100*sumeuk_diso[i]/numdiso_euk,1))               


        
    numpy.save(output_dir+"sumbac_medGC_helix.npy",sumbac_helix)
    numpy.save(output_dir+"sumarch_medGC_helix.npy",sumarch_helix)
    numpy.save(output_dir+"sumeuk_medGC_helix.npy",sumeuk_helix)
    numpy.save(output_dir+"sumbac_medGC_sheet.npy",sumbac_sheet)
    numpy.save(output_dir+"sumarch_medGC_sheet.npy",sumarch_sheet)
    numpy.save(output_dir+"sumeuk_medGC_sheet.npy",sumeuk_sheet)
    numpy.save(output_dir+"sumbac_medGC_loop.npy",sumbac_loop)
    numpy.save(output_dir+"sumarch_medGC_loop.npy",sumarch_loop)
    numpy.save(output_dir+"sumeuk_medGC_loop.npy",sumeuk_loop)
    numpy.save(output_dir+"sumbac_medGC_diso.npy",sumbac_diso)
    numpy.save(output_dir+"sumarch_medGC_diso.npy",sumarch_diso)
    numpy.save(output_dir+"sumeuk_medGC_diso.npy",sumeuk_diso)
