#!/usr/bin/env python3
from operator import mul
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import seaborn as sns
import functools
from scipy.stats import ttest_ind
from scipy.stats import mannwhitneyu
from scipy.spatial import distance
from scipy.stats import norm
#from numpy import sqrt, abs, round
import numpy as np
import math as math
from matplotlib import colors
from matplotlib import cm




if __name__ == '__main__':
    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length_translation":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea',
                "full":"Full", "pfam":"Shared domains", "domains":"Specific domains", 
                "linkers":"Linker regions",
                  "All":"All proteins","Unique":"Specific Proteins","Shared":"Shared Proteins","None":"No Domain proteins"
                     ,"numdiso_long":"Num disorder residues (long)"
                     ,"numdiso_short":"Num disorder residues (short)"
                     ,"numseg":"Low complexity residues",
                      'freq_A-shift': '$ALA^*$', 'freq_C-shift': '$CYS^*$', 'freq_E-shift': '$GLU^*$',
                      'freq_D-shift': '$ASP^*$', 'freq_G-shift': '$GLY^*$', 'freq_F-shift': '$PHE^*$',
                      'freq_I-shift': '$ILE^*$', 'freq_H-shift': '$HIS^*$', 'freq_K-shift': '$LYS^*$',
                      'freq_M-shift': '$MET^*$', 'freq_L-shift': '$LEU^*$', 'freq_N-shift': '$ASN^*$',
                      'freq_Q-shift': '$GLN^*$', 'freq_P-shift': '$PRO^*$', 'freq_S-shift':'$SER^*$', 
                      'freq_T-shift': '$THR^*$', 'freq_W-shift': '$TRP^*$', 'freq_V-shift': '$VAL^*$',
                      'freq_Y-shift': '$TYR^*$', 'freq_R-shift': '$ARG^*$' }

    ylabels = {"GC":"Fraction","GC_genomic":"Fraction", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity>", "top-idp":"<TOP-IDP>","topidp-S":"<TOP-IDP-noS>","topidp-SKP":"<TOP-IDP-noSKP>","topidp-SP":"<TOP-IDP-noSP>",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'fraction','taxon_gc':'fraction','taxon_gc_r':'fraction',
    'random':'fraction','iupred_long':'Fraction of AA','length_aa':'<Length>','gc':'fraction',
    'iupred_short':'Fration of residues','iupred_cutoff':'Fraction of residues', 'idp':'<TOP-IDP>','tm':'Fraction of residues',"volatility":"<Volatility>", 'iupred_short':'Fraction of residues','iupred_cutoff':'Fraction of residues','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity>','gc':'Fraction','aa_corr':'Correlation', 'aa_corr_dot':'correlation','aa_corr_r':'Correlation', 'A':'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'Fraction of residues', 'freq_C':
    'Fraction of residues', 'freq_E': 'Fraction of residues', 'freq_D': 'Fraction of residues', 'freq_G': 'Fraction of residues',
    'freq_F': 'Fraction of residues', 'freq_I': 'Fraction of residues', 'freq_H': 'Fraction of residues', 'freq_K':
    'Fraction of residues', 'freq_M': 'Fraction of residues', 'freq_L': 'Fraction of residues', 'freq_N': 'Fraction of residues',
    'freq_Q': 'Fraction of residues', 'freq_P': 'Fraction of residues', 'freq_S': 'Fraction of residues', 'freq_R':
    'Fraction of residues', 'freq_T': 'Fraction of residues', 'freq_W': 'Fraction of residues', 'freq_V': 'Fraction of residues',
    'freq_Y': 'Fraction of residues', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea'
                     ,"numdiso_long":"Num disorder residues (long)"
                     ,"numdiso_short":"Num disorder residues (short)"
                     ,"numseg":"Low complexity residues"                ,
                     'freq_A-shift': 'Shift', 'freq_C-shift': 'Shift', 'freq_E-shift': 'Shift',
                      'freq_D-shift': 'Shift', 'freq_G-shift': 'Shift', 'freq_F-shift': 'Shift',
                      'freq_I-shift': 'Shift', 'freq_H-shift': 'Shift', 'freq_K-shift': 'Shift',
                      'freq_M-shift': 'Shift', 'freq_L-shift': 'Shift', 'freq_N-shift': 'Shift',
                      'freq_Q-shift': 'Shift', 'freq_P-shift': 'Shift', 'freq_S-shift':'SER', 'freq_R-shift': 'Shift',
                      'freq_T-shift': 'Shift', 'freq_W-shift':        'Shift', 'freq_V-shift': 'Shift',
                      'freq_Y-shift': 'Shift' }

    data_dir="../data/"
    figures_dir="../figures_medGC/"
    # the properties to plot
    properties_x = ["GC_genomic"]
    #properties_x = ["GC", "GC3", "GC_genomic"]

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks],sort=True)

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']
    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]

    
    pfam_sets = ["full", "pfam", "domains", "linkers"]
    uniprot_sets = ["All","None","Shared","Unique"]
    datasets=pfam_sets+uniprot_sets
        
    minGC=20.
    maxGC=60.
    dfs={}
    for d in datasets:
        dfs[d]=pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-GC-" + d+ ".csv").dropna(axis=0)
        dfs[d]=dfs[d].loc[(dfs[d]["GC_genomic"].astype(float) > minGC) & (dfs[d]["GC_genomic"].astype(float)<maxGC) ]
        
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
    df=dfs["All"]
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

    #######  HEATMAP
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    data = []
    y_labels = []

    datasets=["None","Shared","Unique", "pfam", "domains", "linkers"]
    domsets=["pfam", "domains", "linkers"]
    
    for kingdom in ["Eukaryota", "Bacteria"]:
        for ty in datasets:
            if ty != "full": # Just for checking we include full as well
                d = []
                for prop in props:
                    length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                    value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                    d += [np.mean(value)]
                    
                    #print (kingdom, ty,prop, value)
                    
                y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
                
                data += [d]
                

        

    sns.set()
    sns.set(font_scale=0.8)
    
    np.savetxt("freq.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')


    sns.clustermap(data, col_cluster=False,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')

    plt.savefig(figures_dir + "aa_freq_heatmap.eps",bbox_inches="tight")

    #######  HEATMAP
    props = ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    data = []
    y_labels = []
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
        for ty in datasets:
            if ty != "full": # Just for checking we include fill as well
                d = []
                for prop in props:
                    length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                    value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                    d += [np.mean(value)]
                    
                    #print (kingdom, ty,prop, value)
                    
                y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
                
                data += [d]
                

        

    sns.set()
    sns.set(font_scale=0.8)
    
    #np.savetxt("freq.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')

    # We change to euclidean here as almost all Bacterial groups are 0.
    sns.clustermap(data, col_cluster=True,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="bwr",annot=False, annot_kws={"size": 3},metric='euclidean')

    plt.savefig(figures_dir + "aa_freq-shift_heatmap.eps",bbox_inches="tight")

    #######  HEATMAP - only domains
    props = ["freq_" + aa  for aa in aas_sorted_topidp]
    data = []
    y_labels = []
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
        for ty in domsets:
            if ty != "full": # Just for checking we include fill as well
                d = []
                for prop in props:
                    length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                    value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                    d += [np.mean(value)]
                    
                    #print (kingdom, ty,prop, value)
                    
                y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
                
                data += [d]
                

        

    sns.set()
    sns.set(font_scale=1.2)
    
    #np.savetxt("freq.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')

    # We change to euclidean here as almost all Bacterial groups are 0.
    sns.clustermap(data, col_cluster=True,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')

    plt.savefig(figures_dir + "aa_freq_dom_heatmap.eps",bbox_inches="tight")

    #######  HEATMAP - only domains -shift
    props = ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    data = []
    y_labels = []
    for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
        for ty in domsets:
            if ty != "full": # Just for checking we include fill as well
                d = []
                for prop in props:
                    length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                    value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                    d += [np.mean(value)]
                    
                    #print (kingdom, ty,prop, value)
                    
                y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
                
                data += [d]
                

        

    sns.set()
    sns.set(font_scale=1.2)
    
    #np.savetxt("freq.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')

    # We change to euclidean here as almost all Bacterial groups are 0.
    sns.clustermap(data, col_cluster=True,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="bwr",annot=False, annot_kws={"size": 3},metric='euclidean')

    plt.savefig(figures_dir + "aa_freq_dom-shift_heatmap.eps",bbox_inches="tight")

    #################################################
    propsA = ["freq_" + aa for aa in aas_sorted_topidp]
    propsB = ["freq_" + aa + "-shift" for aa in aas_sorted_topidp]
    
    #    kingdoms= ["Bacteria","Archaea","Eukaryota","Viruses"]
    kingdoms= ["Bacteria","Archaea","Eukaryota","Viruses"]
    phyla = df.phylum.dropna().unique()
    
    kingdom2color = {}
    for k, c in zip(kingdoms, ['#E20048', '#5064d0','#00B060',  '#FFA500']):
        kingdom2color[k] = c
        

    phylum2color = {}
    phylum2color_gc = {}

    for phylum in phyla:
        tdf = df.loc[df.phylum == phylum]
        kingdom = tdf.kingdom.iloc[0]
        col = kingdom2color[kingdom]
        phylum2color[phylum] = col
        
        gc = pd.np.mean(pd.np.array([float(f) for f in tdf.GC.replace("-",pd.np.nan).dropna()]))

        # color according to GC only
        # print (phylum,gc)
        if gc > 50:
            col = "darkred"
        elif gc > 45:
            col = "pink"
        elif gc > 40:
            col = "lightgreen"
        else:
            col = "darkgreen"
        cmap = cm.get_cmap('viridis')
        max=71
        min=34
        col=cmap( (gc-min)/(max-min))
        phylum2color_gc[phylum] = col


    
    data = {}

    props=propsA
    for phylum in phyla:

        tdf = df.loc[df.phylum == phylum]
        if len(tdf.length) > 10:
            d = []
            for prop in props:
                value = float(tdf[prop].mean())
                d += [value]
                
            #data[list(tdf.kingdom)[0] + " - " + phylum] = d
            data[phylum] = d


    data_dic = {}
    for s1 in data.keys():
        data_dic[s1] = {}
        for s2 in data.keys():
            #data_dic[s1][s2] = round(pd.np.corrcoef(data[s1], data[s2])[0, 1],3)
            data_dic[s1][s2] = 1/(1+distance.euclidean(data[s1],data[s2]))

    df_r = pd.DataFrame(data_dic)



    # plot a clustered heatmap of all-vs-all phyla

    colors_gc = [phylum2color_gc[p] for p in df_r.columns]
    for method in ["single","complete","average","weighted"]: # ,"centroid","ward"]:
        fig, ax = plt.subplots(figsize=(6,6))

        sns.set(font_scale=0.45)
        g=sns.clustermap(df_r,row_colors=colors_gc,cmap="Purples",method=method,metric='correlation')
        g.ax_row_colors.set_title("GC",fontsize=11)

        for lbl in g.ax_heatmap.yaxis.get_ticklabels():
            #lbl.set_color(phylum2color[lbl.get_text()])
            lbl.set_bbox(dict(facecolor=phylum2color[lbl.get_text()], alpha=1, edgecolor="white",
                            mutation_scale=2,mutation_aspect=0.2))
            
            
        # add a separate axis for the legend
        # left, bottom, width, height
        rect = 1.02,0.35,0.25,0.4 
        ax_legend = g.fig.add_axes(rect, label='axes1',facecolor="white")
        ax_legend.set_xticks([])
        ax_legend.set_yticks([])
        

        ptcs = []
#        for l,c in zip(["Bacteria","Archaea","Eukaryota","Viruses"],['#E20048', '#5064d0','#00B060',  '#FFA500']):
        for l,c in zip(["Bacteria","Archaea","Eukaryota"],['#E20048','#00B060']):
            ptcs += [mpatches.Patch(color=c, label=l)]
            
        ax_legend.legend(handles=ptcs,loc="upper left",fontsize=20)
        

        # add a separate axis for the GC viridis colorbar
        rect = 1.05,0.15,0.05,0.3 
        ax_cbar = g.fig.add_axes(rect, label='axes2',facecolor="white")
        ax_cbar.set_xticks([])
        ax_cbar.set_yticks([])
        
        ax_cbar.set_title("GC%",fontsize=14)
        '''                
        
        import matplotlib as mpl
        mpl.rc('ytick', labelsize=20)
        mpl.rc('xtick', labelsize=20)
        cmap = mpl.cm.viridis
        norm = mpl.colors.Normalize(vmin=34, vmax=71)
        cb1 = mpl.colorbar.ColorbarBase(ax_cbar, cmap=cmap,
                                        norm=norm,
                                        orientation='vertical')

        '''
        ax_cbar.tick_params(axis='both', which='major', labelsize=10)
        
        plt.savefig(figures_dir+"/phyla_heatmap-"+method+".eps",bbox_inches="tight")
    

    


