#!/usr/bin/env python3
from operator import mul
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import seaborn as sns
import functools
import numpy as np

data_dir="../data/"
figures_dir="../figures_new/"
genomes_overview_file = data_dir + "overview.txt"
genomes_euks_file = data_dir + "eukaryotes.txt"
genomes_proks_file = data_dir + "prokaryotes.txt"
    
df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
df_reference = pd.concat([df_euks, df_proks],sort=True)

taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]
    

taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
taxid2kingdom = {}
    
for t in taxid2group:
    gr = taxid2group[t]
    taxid2kingdom[t] = group2kingdom[gr]
    
taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']
mycotaxa = []
for t in name2taxid:
    tax = t.split()[0]
    if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
        mycotaxa += [name2taxid[t]]


datasets=["All"]
for d in datasets:
    df=pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-GC-" + d+ ".csv").dropna(axis=0)
good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

colors= {"Bacteria":'#E20048',"Archaea":'#5064d0',"Eukaryota":'#00B060',"Viruses":'#FFA500'}
genomes = df.loc[(df.taxon_id.isin(good_taxa))]["GC_genomic"]
proteins = df.loc[(df.taxon_id.isin(good_taxa))]["count_protein"]
print ("genomes:",genomes.count(),"Proteins:",proteins.sum())

fig, ax = plt.subplots(figsize=(6,6))
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for d in datasets:
        data = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))]["GC_genomic"]
        ax = sns.distplot(data,hist=False,color=colors[kingdom])
        print(kingdom, data.count(),round(np.mean(data),1),round(np.std(data),1))
ax.set_ylabel("Density")
ax.set_xlabel("GC content")
ax.set_title("Distribution of GC content in all genomes")
figname=figures_dir+"GC-distribution.eps"
plt.savefig(figname,bbox_inches="tight")

minGC=20.
maxGC=60.
df=df.loc[(df["GC_genomic"].astype(float) > minGC) & (df["GC_genomic"].astype(float)<maxGC) ]
genomes = df.loc[(df.taxon_id.isin(good_taxa))]["GC_genomic"]
proteins = df.loc[(df.taxon_id.isin(good_taxa))]["count_protein"]
print ("genomes:",genomes.count(),"Proteins:",proteins.sum())

fig, ax = plt.subplots(figsize=(6,6))
for kingdom in ["Eukaryota", "Bacteria","Archaea"]:
    for d in datasets:
        data = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))]["GC_genomic"]
        ax = sns.distplot(data,hist=False,color=colors[kingdom])
        print(kingdom, data.count(),round(np.mean(data),1),round(np.std(data),1))
ax.set_ylabel("Density")
ax.set_xlabel("GC content")
ax.set_title("Distribution of GC content in genomes (final set)")
figname=figures_dir+"GC-distribution-lim.eps"
plt.savefig(figname,bbox_inches="tight")



    
