#!/usr/bin/env python
import matplotlib
matplotlib.use('Agg')
import pandas as pd
import numpy as np
import os
import re
import seaborn as sns
import matplotlib.pyplot as plt

aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
               'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
               'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}

dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)", "top-idp":"TOP-IDP", "topidp-S":"TOP-IDP - S","topidp-SKP":"TOP-IDP - SKP","topidp-SP":"TOP-IDP - SP",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'GC% (third nucleotide)','taxon_gc':'GC% (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
    'random':'Expected','iupred_long':'IUpred long (%AA)','length_aa':'<Length> (AA)','gc':'GC%',
    'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)', 'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM (%AA)',"volatility":"<Volatility>", 'iupred_short':'IUpred short (%AA)','iupred_cutoff':'IUpred cutoff (%AA)','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity> (Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with  expected', 'aa_corr_dot':'AA freq\ncorrelation with   expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A':
    'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P':'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'ALA', 'freq_C':
    'CYS', 'freq_E': 'GLU', 'freq_D': 'ASP', 'freq_G': 'GLY',
    'freq_F': 'PHE', 'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K':
    'LYS', 'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
    'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S': 'SER', 'freq_R':
    'ARG', 'freq_T': 'THR', 'freq_W': 'TRP', 'freq_V': 'VAL',
    'freq_Y': 'TYR', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}
        

sns.set()
sns.set(font_scale=1.4)

pfam_list = []
if os.path.isdir("../data/substmat/"):
    input_dir="../data/substmat/"
else:
    input_dir="../data/"
data_dir="../data/"
figures_dir="../figures_medGC/"

        
num_e=np.zeros(20)
num_b=np.zeros(20)
num_ee=np.zeros(20)
num_bb=np.zeros(20)

bac_helix=np.load(input_dir+"sumbac_medGC_helix.npy")
arch_helix=np.load(input_dir+"sumarch_medGC_helix.npy")
euk_helix=np.load(input_dir+"sumeuk_medGC_helix.npy")
bac_sheet=np.load(input_dir+"sumbac_medGC_sheet.npy")
arch_sheet=np.load(input_dir+"sumarch_medGC_sheet.npy")
euk_sheet=np.load(input_dir+"sumeuk_medGC_sheet.npy")
bac_loop=np.load(input_dir+"sumbac_medGC_loop.npy")
arch_loop=np.load(input_dir+"sumarch_medGC_loop.npy")
euk_loop=np.load(input_dir+"sumeuk_medGC_loop.npy")
bac_diso=np.load(input_dir+"sumbac_medGC_diso.npy")
arch_diso=np.load(input_dir+"sumarch_medGC_diso.npy")
euk_diso=np.load(input_dir+"sumeuk_medGC_diso.npy")

sumbac_helix=sum(bac_helix)
sumarch_helix=sum(arch_helix)
sumeuk_helix=sum(euk_helix)
sumbac_sheet=sum(bac_sheet)
sumarch_sheet=sum(arch_sheet)
sumeuk_sheet=sum(euk_sheet)
sumbac_loop=sum(bac_loop)
sumarch_loop=sum(arch_loop)
sumeuk_loop=sum(euk_loop)
sumbac_diso=sum(bac_diso)
sumarch_diso=sum(arch_diso)
sumeuk_diso=sum(euk_diso)

freqbac_helix=np.divide(bac_helix,sumbac_helix)
freqarch_helix=np.divide(arch_helix,sumarch_helix)
freqeuk_helix=np.divide(euk_helix,sumeuk_helix)
freqbac_sheet=np.divide(bac_sheet,sumbac_sheet)
freqarch_sheet=np.divide(arch_sheet,sumarch_sheet)
freqeuk_sheet=np.divide(euk_sheet,sumeuk_sheet)
freqbac_loop=np.divide(bac_loop,sumbac_loop)
freqarch_loop=np.divide(arch_loop,sumarch_loop)
freqeuk_loop=np.divide(euk_loop,sumeuk_loop)
freqbac_diso=np.divide(bac_diso,sumbac_diso)
freqarch_diso=np.divide(arch_diso,sumarch_diso)
freqeuk_diso=np.divide(euk_diso,sumeuk_diso)


# Helix
fig, ax = plt.subplots()
index = np.arange(20)
bar_width = 0.25
opacity = 0.4
rects1 = plt.bar(index, freqbac_helix, bar_width,
     alpha=opacity,color='#E20048',
     label='Bacteria',edgecolor='black')
rects3 = plt.bar(index + bar_width, freqarch_helix , bar_width,
     alpha=opacity, color='#5064d0',
    label='Archaea',edgecolor='black')
rects2 = plt.bar(index + 2*bar_width, freqeuk_helix , bar_width,
     alpha=opacity, color='#00B060',
    label='Eukaryota',edgecolor='black')
ax.set_facecolor('white')
plt.xlabel('Amino Acid')
plt.ylabel('Frequency')
plt.title('Frequency of amino acids in helix ')
plt.xticks(index + bar_width / 1, aas)
plt.legend()
ax.yaxis.grid()
ax.grid(color='grey', linestyle='-', linewidth=1)
ax.xaxis.grid(False)
plt.tight_layout()
#plt.show()
plt.savefig(figures_dir + "helix-freq.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
plt.close()

# Sheet
fig, ax = plt.subplots()
index = np.arange(20)
bar_width = 0.25
opacity = 0.4
rects1 = plt.bar(index, freqbac_sheet, bar_width,
     alpha=opacity,color='#E20048',
     label='Bacteria',edgecolor='black')
rects3 = plt.bar(index + bar_width, freqarch_sheet , bar_width,
     alpha=opacity, color='#5064d0',
    label='Archaea',edgecolor='black')
rects2 = plt.bar(index + 2*bar_width, freqeuk_sheet , bar_width,
     alpha=opacity, color='#00B060',
    label='Eukaryota',edgecolor='black')
ax.set_facecolor('white')
plt.xlabel('Amino Acid')
plt.ylabel('Frequency')
plt.title('Frequency of amino acids in sheet ')
plt.xticks(index + bar_width / 1, aas)
plt.legend()
ax.yaxis.grid()
ax.grid(color='grey', linestyle='-', linewidth=1)
ax.xaxis.grid(False)
plt.tight_layout()
#plt.show()
plt.savefig(figures_dir + "sheet-freq.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
plt.close()

# Loop
fig, ax = plt.subplots()
index = np.arange(20)
bar_width = 0.25
opacity = 0.4
rects1 = plt.bar(index, freqbac_loop, bar_width,
     alpha=opacity,color='#E20048',
     label='Bacteria',edgecolor='black')
rects3 = plt.bar(index + bar_width, freqarch_loop , bar_width,
     alpha=opacity, color='#5064d0',
    label='Archaea',edgecolor='black')
rects2 = plt.bar(index + 2*bar_width, freqeuk_loop , bar_width,
     alpha=opacity, color='#00B060',
    label='Eukaryota',edgecolor='black')
ax.set_facecolor('white')
plt.xlabel('Amino Acid')
plt.ylabel('Frequency')
plt.title('Frequency of amino acids in loop ')
plt.xticks(index + bar_width / 1, aas)
plt.legend()
ax.yaxis.grid()
ax.grid(color='grey', linestyle='-', linewidth=1)
ax.xaxis.grid(False)
plt.tight_layout()
#plt.show()
plt.savefig(figures_dir + "loop-freq.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
plt.close()

# Diso
fig, ax = plt.subplots()
index = np.arange(20)
bar_width = 0.25
opacity = 0.4
rects1 = plt.bar(index, freqbac_diso, bar_width,
     alpha=opacity,color='#E20048',
     label='Bacteria',edgecolor='black')
rects3 = plt.bar(index + bar_width, freqarch_diso , bar_width,
     alpha=opacity, color='#5064d0',
    label='Archaea',edgecolor='black')
rects2 = plt.bar(index + 2*bar_width, freqeuk_diso , bar_width,
     alpha=opacity, color='#00B060',
    label='Eukaryota',edgecolor='black')
ax.set_facecolor('white')
plt.xlabel('Amino Acid')
plt.ylabel('Frequency')
plt.title('Frequency of amino acids in disordered regions ')
plt.xticks(index + bar_width / 1, aas)
plt.legend()
ax.yaxis.grid()
ax.grid(color='grey', linestyle='-', linewidth=1)
ax.xaxis.grid(False)
plt.tight_layout()
#plt.show()
plt.savefig(figures_dir + "diso-freq.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
plt.close()


# Tyrosine is missing ???

secstr=['helix','sheet','loop','diso']
for i in range(20):
    euk=np.zeros(4)
    bac=np.zeros(4)
    arch=np.zeros(4)
    euk[0]=freqeuk_helix[i]
    bac[0]=freqbac_helix[i]
    arch[0]=freqarch_helix[i]
    euk[1]=freqeuk_sheet[i]
    bac[1]=freqbac_sheet[i]
    arch[1]=freqarch_sheet[i]
    euk[2]=freqeuk_loop[i]
    bac[2]=freqbac_loop[i]
    arch[2]=freqarch_loop[i]
    euk[3]=freqeuk_diso[i]
    bac[3]=freqbac_diso[i]
    arch[3]=freqarch_diso[i]
    fig, ax = plt.subplots()
    index = np.arange(4)
    bar_width = 0.25
    
    opacity = 0.4
    rects1 = plt.bar(index, bac, bar_width,
        alpha=opacity,color='#E20048',
        label='Bacteria',edgecolor='black')
    rects3 = plt.bar(index + bar_width, arch , bar_width,
        alpha=opacity, color='#5064d0',
        label='Archaea',edgecolor='black')
    rects2 = plt.bar(index + 2*bar_width, euk , bar_width,
        alpha=opacity, color='#00B060',
        label='Eukaryota',edgecolor='black')
    ax.set_facecolor('white')
    plt.xlabel('Secondary Structure')
    plt.ylabel('Frequency')
    plt.title('Frequency of  ' + dic_labels[aas[i]] )
    plt.xticks(index + bar_width / 1, secstr)
    plt.legend()
    ax.yaxis.grid()
    ax.grid(color='grey', linestyle='-', linewidth=1)
    ax.xaxis.grid(False)
    plt.tight_layout()
    #plt.show()
    plt.savefig(figures_dir + "secstr-"+aas[i]+"-freq.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close()

secstr=['helix','sheet','loop','diso']
for i in range(20):
    euk=np.zeros(4)
    bac=np.zeros(4)
    arch=np.zeros(4)
    euk[0]=freqeuk_helix[i]-freqbac_helix[i]
    bac[0]=freqbac_helix[i]
    arch[0]=freqarch_helix[i]-freqbac_helix[i]
    euk[1]=freqeuk_sheet[i]-freqbac_sheet[i]
    bac[1]=freqbac_sheet[i]
    arch[1]=freqarch_sheet[i]-freqbac_sheet[i]
    euk[2]=freqeuk_loop[i]-freqbac_loop[i]
    bac[2]=freqbac_loop[i]
    arch[2]=freqarch_loop[i]-freqbac_loop[i]
    euk[3]=freqeuk_diso[i]-freqbac_diso[i]
    bac[3]=freqbac_diso[i]
    arch[3]=freqarch_diso[i]-freqbac_diso[i]
    fig, ax = plt.subplots()
    index = np.arange(4)
    bar_width = 0.35
    ylim=0.02
    ax.set_ylim([-ylim, ylim])
    
    opacity = 0.4
    #rects1 = plt.bar(index, bac, bar_width,
    #    alpha=opacity,color='#E20048',
    #    label='Bacteria',edgecolor='black')
    rects3 = plt.bar(index , arch , bar_width,
        alpha=opacity, color='#5064d0',
        label='Archaea',edgecolor='black')
    rects2 = plt.bar(index + bar_width, euk , bar_width,
        alpha=opacity, color='#00B060',
        label='Eukaryota',edgecolor='black')
    ax.set_facecolor('white')
    plt.xlabel('Secondary Structure')
    plt.ylabel('Frequency')
    plt.title('Frequency shift of  ' + dic_labels[aas[i]] )
    plt.xticks(index + bar_width / 1, secstr)
    plt.legend()
    ax.yaxis.grid()
    ax.grid(color='grey', linestyle='-', linewidth=1)
    ax.xaxis.grid(False)
    plt.tight_layout()
    #plt.show()
    plt.savefig(figures_dir + "secstr-"+aas[i]+"-freq-shift.eps",transparent=False,bbox_inches = None)# ,edgecolor='none',FrameOn=True,bbox_inches = "tight")
    plt.close()
