#!/usr/bin/env python3

import pickle
import os
import pandas as pd
import csv

aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
        'hessa':"Hydrophobicity-Hessa",
        "top-idp":"TOP-IDP", "ss_alpha":"<Alpha-propensity>","ss_beta":"<Beta-propensity>","ss_coil":"<Coil-propensity>",
        'ss_turn':"<Turn-propensity>","length_translation":"Length(AA)",
                  'gc_third':'GC3% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred-long(%AA)','length_aa':'<Length>(AA)','gc':'GC%',
                'iupred_short':'IUpred-short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'SEG', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea',
                "full":"Full", "pfam":"Shared domains", "domains":"Exclusive domains", 
                "linkers":"Linkers",
                "N-linkers":"N-Linkers",
                "M-linkers":"M-Linkers",
                "C-linkers":"C-Linkers",
                  "All":"All proteins","Unique":"Specific Proteins","Shared":"Shared Proteins","None":"No Domain proteins"
                     ,"numdiso_long":"Num disorder residues (long)"
                     ,"numdiso_short":"Num disorder residues (short)"
                     ,"numseg":"Low complexity residues",
                  }
                

data_dir="../data/"
# open the three reference files
genomes_overview_file = data_dir + "overview.txt"
genomes_euks_file = data_dir + "eukaryotes.txt"
genomes_proks_file = data_dir + "prokaryotes.txt"
    
df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")
df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
df_reference = pd.concat([df_euks, df_proks],sort=True)

taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
taxid2kingdom = {}
    
for t in taxid2group:
    gr = taxid2group[t]
    taxid2kingdom[t] = group2kingdom[gr]
    
taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']

                
props = ["iupred_long","iupred_short", "seg","top-idp","hessa","length_translation"]
props += ["freq_" + aa for aa in aas]
props += ["ss_alpha", "ss_beta", "ss_coil", "ss_turn"]


#diclen = {"Eukaryota":4932465,"Bacteria": 8726710,"Archaea":1,"Viruses":1}


uniprotset=["All","Unique","Shared","None"]
pfamset=["full","pfam","domains","linkers","N-linkers","M-linkers","C-linkers"]
fullset=uniprotset+pfamset
# We add the new files into the same format

for ty in uniprotset:
    out_dic_fname = "props_dic_"+ty+".pickle"
    if not os.path.exists(out_dic_fname):
        df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv")
        #df["length_translation"]=df.length
        dic_res = {}
        dic_res[ty] = {}
        for kingdom in ["Eukaryota", "Bacteria","Archaea","Viruses"]:
            #print "\t", kingdom
            dic_res[ty][kingdom] = {}
            kdf = df.loc[df.kingdom == kingdom]

            dic_res[ty][kingdom]["total_residues"] = int(kdf["length_translation"].sum())
            dic_res[ty][kingdom]["total_proteins"] = int(kdf["count_protein"].sum())

            for prop in props:
                if (str(prop)  == "length_translation"):
                    dic_res[ty][kingdom][prop] = kdf[prop].sum()/float(dic_res[ty][kingdom]["total_proteins"])
                    dic_res[ty][kingdom][prop+"_sem"] = kdf[prop].sem()
                else:
                    dic_res[ty][kingdom][prop] = kdf[prop].sum()/float(dic_res[ty][kingdom]["total_residues"])
                    dic_res[ty][kingdom][prop+"_sem"] = kdf[prop].sem()/float(dic_res[ty][kingdom]["total_residues"])
            # This was with mean (but the frequencies differed compared to Pfam data)
            #for prop in props:
            #    dic_res[ty][kingdom][prop] = kdf[prop].mean()
            #    dic_res[ty][kingdom][prop+"_sem"] = kdf[prop].sem()
                    
        #print (ty,dic_res)
        with open(out_dic_fname, 'wb') as handle:
            pickle.dump(dic_res, handle, protocol=pickle.HIGHEST_PROTOCOL)

ty="full"
out_dic_fname = "props_dic_"+ty+".pickle"
diclen={}
for kingdom in ["Eukaryota", "Bacteria","Archaea","Viruses"]:
    df = pd.read_csv(data_dir + "df_" + ty+ "_annotation_by_species.csv")
    diclen[kingdom]=df.loc[df.kingdom == kingdom]["count_protein"].sum()
            
for ty in pfamset:
    out_dic_fname = "props_dic_"+ty+".pickle"
    dic_res = {}
    df = pd.read_csv(data_dir + "df_" + ty+ "_annotation_by_species.csv")
    dic_res[ty] = {}
    for kingdom in ["Eukaryota", "Bacteria","Archaea","Viruses"]:
        #print "\t", kingdom
        dic_res[ty][kingdom] = {}
        kdf = df.loc[df.kingdom == kingdom]

        dic_res[ty][kingdom]["total_residues"] = int(kdf["length_translation"].sum())
        dic_res[ty][kingdom]["total_proteins"] = int(kdf["count_protein"].sum())
        print (ty,kingdom,dic_res[ty][kingdom]["total_residues"], dic_res[ty][kingdom]["total_proteins"] ,diclen[kingdom])
        for prop in props:
            if (str(prop)  == "length_translation"):
                # The number of proteins is actuallt wrong in the Pfam entries. not really sure why
                dic_res[ty][kingdom][prop] = kdf[prop].sum()/float(diclen[kingdom])
                #dic_res[ty][kingdom][prop] = kdf[prop].sum()/float(dic_res[ty][kingdom]["total_proteins"])
                dic_res[ty][kingdom][prop+"_sem"] = kdf[prop].sem()/float(diclen[kingdom])
                #dic_res[ty][kingdom][prop+"_sem"] = kdf[prop].sem()/float(dic_res[ty][kingdom]["total_proteins"])
            else:
                dic_res[ty][kingdom][prop] = kdf[prop].sum()/float(dic_res[ty][kingdom]["total_residues"])
                dic_res[ty][kingdom][prop+"_sem"] = kdf[prop].sem()/float(dic_res[ty][kingdom]["total_residues"])

        #print (ty,dic_res)
    with open(out_dic_fname, 'wb') as handle:
        pickle.dump(dic_res, handle, protocol=pickle.HIGHEST_PROTOCOL)




dic_res = {}
for ty in fullset:

    out_dic_fname = "props_dic_"+ty+".pickle"
    
    with open(out_dic_fname, 'rb') as handle:
        dic_res.update(pickle.load(handle))


print ("\\begin{tabular}[cccc|cccc|cccc|cccc]")
for prop in props:
    head = "Property" 
    for kingdom in ["Eukaryota", "Bacteria","Archaea","Viruses"]:
        for ty in fullset:
            head += " & " + dic_labels[ty]
    
print (head, "\\\\")

for prop in props:
    line = []
    line += [dic_labels[prop].replace("%","\\%").replace("<","$<$").replace(">","$>$")]
    #print dic_labels[prop]," & ",
    
    for kingdom in ["Eukaryota", "Bacteria","Archaea","Viruses"]:
        #print kingdom, prop
        #print "Total residues:", dic_res["full"][kingdom]["total_residues"] 
        #~ print "Dataset,", ",".join([prop+" residues", "Total residues", prop+"% (local)", prop+"% (global)"])
        for ty in fullset:
            
            if prop in ["ss_alpha", "ss_beta", "ss_coil", "ss_turn", "top-idp", "hessa","length_translation"]:
                #~ print ty.upper() ,",",dic_res[ty][kingdom][prop],",",dic_res[ty][kingdom][prop+"_sem"]
                #~ print round(dic_res[ty][kingdom][prop], 2)," & ",
                line += [round(dic_res[ty][kingdom][prop], 1)]
                
            #elif prop in ["length_translation"]:
                #~ print round(float(dic_res[ty][kingdom][prop])/15208818.0,2)," & ",
            #    diclen = {"Eukaryota":4932465,"Bacteria": 8726710}
            #    line += [round(float(dic_res[ty][kingdom][prop])/float(diclen[kingdom]),2)]
            else:
                #print dic_res[ty][kingdom]["total_residues"]
                #print float(dic_res[ty][kingdom][prop])
                
                #perc_glob = float(dic_res[ty][kingdom][prop])/float(dic_res["full"][kingdom]["total_residues"])*100.0
                perc_loc = float(dic_res[ty][kingdom][prop])*100.0

                #~ print ty.upper() ,",", int(dic_res[ty][kingdom][prop]),",", int(dic_res[ty][kingdom]["total_residues"]),",", round(perc_loc, 3),",", round(perc_glob,3)
                line += [round(perc_loc, 1)]
    print (" & ".join([str(x) for x in line]), "\\\\")
            

print ("\\end{tabular}")
