#!/usr/bin/env python3
from __future__ import print_function



def darken(col):
    c = colors.hex2color(col)
    nc = []
    for e in c:
        ne = e-0.4
        if ne < 0: ne = 0
        nc += [ne]
    return nc

    

def twoSampZ(X1, X2, mudiff, sd1, sd2, n1, n2):
    pooledSE = sqrt(sd1**2/n1 + sd2**2/n2)
    z = ((X1 - X2) - mudiff)/pooledSE
    pval = 2*(1 - norm.cdf(abs(z)))
    return round(z, 3), round(pval, 4)

def get_topidp(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp[aa]*data["freq_"+aa]
    return score

def get_topidp_noser(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noser[aa]*data["freq_"+aa]
    return score

def get_topidp_noserpro(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noserpro[aa]*data["freq_"+aa]
    return score

def get_topidp_noserlyspro(data):
    score=0.
    for aa in aas_sorted_topidp:
        score+=top_idp_noserlyspro[aa]*data["freq_"+aa]
    return score
    
def get_aa_freq(aa,gc):
    codons = aa2codons[aa]
    prob_tot = []
    for codon in codons:
        prob_codon = []
        for N in codon:
            prob = gc/2
            if N in {"A","T","U"}:
                prob = 0.5-(gc/2)
            prob_codon += [prob]
        #print(codon, prob_codon,functools.reduce(mul,prob_codon))
        prob_tot += [functools.reduce(mul,prob_codon)] # Use functools.reduce() if you really need it; however, 99 percent of the time an explicit for loop is more readable
    prob_tot = sum(prob_tot)
    return prob_tot



##########################################################################################################################

# for each amino acid, and separately for eukarya and bacteria, plot its frequency versus GC% for 4 curves:
# 1- average freq
# 2- a linear fit of the data
# 3- the frequency as it should be if only dependent on GC
# 4- a model function (w * prob_tot + k) with estimated parameters W and K
# note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

# w and k are the parameters to optimize
def my_func(x,y,w,k):
    global aminoacid
    codons = aa2codons[aminoacid]
    res = []
    for gc in x:
        prob_tot = []
        for codon in codons:
            prob_codon = []
            for N in codon:
                prob = gc/2
                if N in {"A","T","U"}:
                    prob = 0.5-(gc/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            prob_tot += [functools.reduce(mul,prob_codon)]
        prob_tot = sum(prob_tot)
        res += [w * prob_tot + k]
    return res


def get_error(obs,exp):
    err = []
    for o,e in zip(obs,exp):
        err += [abs(o - e)]
    return np.mean(err)


def clamp(value, min_v, max_v):
    return max(min(value, max_v), min_v)

##########################################################################################################################

# define a function to plot a bar plot of the difference between AA frequencies in two datasets
# d1, d2 = the two dataset (could be the same dataset); datasets are "dataset","dataset-eggnog","dataset-pfam","dataset-pfam-full"
# k1, k2 = the kingdoms: can be either "Eukaryota" or "Bacteria"


def bar_plot_s1_vs_s2(d1,d2,k1,k2,weighted=False, uselog = False):
    fig, ax = plt.subplots(figsize=(6.,3))
    data_aa = []
    for aa in aas_sorted_topidp:
        prop = "freq_" + aa
        prop_values = {}
        # We have changed this to do mean before summation - if not different numbers than in Z-test.
        length_1 = dfs[d1].loc[(dfs[d1].kingdom == k1) & (dfs[d1].taxon_id.isin(good_taxa))]["length_translation"]
        length_2 = dfs[d2].loc[(dfs[d2].kingdom == k2) & (dfs[d2].taxon_id.isin(good_taxa))]["length_translation"]
        prop_values_1 = 100*np.mean(dfs[d1].loc[(dfs[d1].kingdom == k1) & (dfs[d1].taxon_id.isin(good_taxa))][prop]/length_1)
        prop_values_2 = 100*np.mean(dfs[d2].loc[(dfs[d2].kingdom == k2) & (dfs[d2].taxon_id.isin(good_taxa))][prop]/length_2)
        if uselog:
            value = np.log((prop_values_1)/(prop_values_2))
        else:
            value = prop_values_1-prop_values_2
        if weighted:
            value *= top_idp[aa]
        data_aa += [value]
    ax.bar(range(len(data_aa)),data_aa, color="red")
    ylim = max([abs(v) for v in ax.get_ylim()])
    if weighted:
        ylim = 1.4
    elif k == l :
        ylim = 1.1
    else:
        ylim = 2.9
    ylim=3.0
    ax.set_ylim([-ylim, ylim])
    ax.set_xticks([0.0 + x for x in range(len(aas_sorted_topidp))])
    ax.set_xticklabels(aas_sorted_topidp)
    if uselog == True:
        ax.set_ylabel("log( freq(" + k1+ ") / freq(" + k2+ ") )")
    else:
        ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
        if weighted:
            ax.set_ylabel("TOP-IDP(" + k1+ " - " + k2+ ")")
        else:
            ax.set_ylabel("freq(" + k1+ ") - freq(" + k2+ ")")
    title = dic_labels[k1] + " (" + dic_labels[d1] + ")" + " vs "  + dic_labels[k2] + " (" + dic_labels[d2] + ")"
    ax.set_title(title)
    ax.yaxis.grid(True)
    ax.xaxis.grid(True)
    if uselog == True:
        if weighted:
            figname = figures_dir + "aa_freq_change_weighted_plot_log_odd_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 +  ".eps"
        else:
            figname = figures_dir + "aa_freq_change_plot_log_odd_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 +  ".eps"
    else:
        if weighted:
            figname = figures_dir + "aa_freq_change_weighted_plot_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + ".eps"
        else:
            figname = figures_dir + "aa_freq_change_plot_" + k1 + "_" + d1 + "_vs_" + k2 + "_" + d2 + ".eps"
    print(figname)
    fig.savefig(figname,dpi=300)
#    fig.show()


def fmt_scientific(x):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)

def running_mean(x, N):
    cumsum = pd.np.cumsum(pd.np.insert(x, 0, 0)) 
    return (cumsum[N:] - cumsum[:-N]) / N 



if __name__ == '__main__':


# coding: utf-8

# In[2]:
    from operator import mul
    import pandas as pd
    import os
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.patches as mpatches
    import seaborn as sns
    import functools
    from scipy.stats import ttest_ind
    from scipy.stats import mannwhitneyu
    from scipy.spatial import distance
    from scipy.stats import norm
    import math as math
    from matplotlib import colors
    from matplotlib import cm
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    #from statsmodels.formula.api import ols
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1
    import seaborn as sns    
    #%matplotlib inline
    
    data_dir = "../data/"
    figures_dir = "../figures_new/"
    kingdoms = ["Bacteria","Eukaryota"]
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks],sort=True)

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # load random genes
    df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")


    # load datasets (per species)
    pfam_sets = ["full", "pfam", "domains", "linkers"]
    uniprot_sets = ["All","None","Shared","Unique"]
    datasets=pfam_sets+uniprot_sets
    #datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

    
    dfs = {}
    for ty in pfam_sets:   
        print (data_dir + "df_" + ty+ "_annotation_by_species.csv")
        df = pd.read_csv(data_dir + "df_" + ty+ "_annotation_by_species.csv").dropna(axis=0)
        df["numdiso_long"]=df.iupred_long*df.length_translation
        df["numdiso_short"]=df.iupred_short*df.length_translation
        df["numseg"]=df.seg*df.length_translation
        df.iupred_long = df.iupred_long*100
        df.iupred_short = df.iupred_short*100
        df.seg = df.seg*100
        dfs[ty] = df

    print ("full",dfs["full"])
    for ty in uniprot_sets:   
        print (data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv")
        df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species-sum-" + ty+ ".csv").dropna(axis=0)
        df["numdiso_long"]=df.iupred_long*df.length_translation
        df["numdiso_short"]=df.iupred_short*df.length_translation
        df["numseg"]=df.seg*df.length_translation
        df.iupred_long = df.iupred_long*100
        df.iupred_short = df.iupred_short*100
        df.seg = df.seg*100
        dfs[ty] = df
    print ("All",dfs["Shared"])
 
    df = dfs["full"]

    '''
    fig, ax = plt.subplots(figsize=(6.,3))
    x=df["length_translation"]
    y=df["freq_K"]/x
    ax.scatter(np.log10(x),y)
    fig.savefig("test.png")
    plt.close()

    for kingdom in ['Eukaryota','Bacteria','Archaea']:
        fig, ax = plt.subplots(figsize=(6.,3))
        x=df.loc[(df.kingdom == kingdom)]["length_translation"]
        y=df.loc[(df.kingdom == kingdom)]["freq_K"]/x
        ax.scatter(np.(x),y)
        fig.savefig(kingdom+".png")
        plt.close()
    sys.exit()
    '''


    # these taxa should be excluded because their genetic code is different, so translation is wrong
    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

    # print some statistics
    #for ds in datasets:
    #    print(ds)
    #    print(dfs[ds].kingdom.dropna().value_counts())
    #    print()
    

    # In[6]:

    # function to get a given AA frequency given the GC content
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']

    codons2aa = {
        'ATA':'I', 'ATC':'I', 'ATT':'I', 'ATG':'M', 'ACA':'T', 'ACC':'T', 'ACG':'T', 'ACT':'T',
        'AAC':'N', 'AAT':'N', 'AAA':'K', 'AAG':'K', 'AGC':'S', 'AGT':'S', 'AGA':'R', 'AGG':'R',
        'CTA':'L', 'CTC':'L', 'CTG':'L', 'CTT':'L', 'CCA':'P', 'CCC':'P', 'CCG':'P', 'CCT':'P',
        'CAC':'H', 'CAT':'H', 'CAA':'Q', 'CAG':'Q', 'CGA':'R', 'CGC':'R', 'CGG':'R', 'CGT':'R',
        'GTA':'V', 'GTC':'V', 'GTG':'V', 'GTT':'V', 'GCA':'A', 'GCC':'A', 'GCG':'A', 'GCT':'A',
        'GAC':'D', 'GAT':'D', 'GAA':'E', 'GAG':'E', 'GGA':'G', 'GGC':'G', 'GGG':'G', 'GGT':'G',
        'TCA':'S', 'TCC':'S', 'TCG':'S', 'TCT':'S', 'TTC':'F', 'TTT':'F', 'TTA':'L', 'TTG':'L',
        'TAC':'Y', 'TAT':'Y', 'TAA':'_', 'TAG':'_', 'TGC':'C', 'TGT':'C', 'TGA':'_', 'TGG':'W'}
        
    aa2codons = {}
    aa2codons["ALL"] = []
    for c in codons2aa:
        aa2codons[codons2aa[c]] = aa2codons.get(codons2aa[c],[]) + [c]
        if c != "_":
            aa2codons["ALL"] += [c]
            
    from operator import mul
    from scipy.optimize import curve_fit
    

    #get_aa_freq("ALL", 0.1)


    # In[7]:

    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length_translation":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea',
                "full":"Full", "pfam":"Shared domains", "domains":"Specific domains", 
                "linkers":"Linker regions",
                  "All":"All proteins","Unique":"Specific Proteins","Shared":"Shared Proteins","None":"No Domain proteins"
                     ,"numdiso_long":"Num disorder resdues (long)"
                     ,"numdiso_short":"Num disorder resdues (short)"
                     ,"numseg":"Low complexity residues"
                      }

    ylabels = {"GC":"Fraction","GC_genomic":"Fraction", 'other':'Intermediate',
    'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity>", "top-idp":"<TOP-IDP>","topidp-S":"<TOP-IDP-noS>","topidp-SKP":"<TOP-IDP-noSKP>","topidp-SP":"<TOP-IDP-noSP>",
    "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>", 'ss_turn':"<Turn propensity>","length_translation":"Length (AA)", 'gc_third':'fraction','taxon_gc':'fraction','taxon_gc_r':'fraction',
    'random':'fraction','iupred_long':'Fraction of AA','length_aa':'<Length>','gc':'fraction',
    'iupred_short':'Fration of residues','iupred_cutoff':'Fraction of residues', 'idp':'<TOP-IDP>','tm':'Fraction of residues',"volatility":"<Volatility>", 'iupred_short':'Fraction of residues','iupred_cutoff':'Fraction of residues','seg':'<SEG>',
    'hydropathy':'<Hydrophobicity>','gc':'Fraction','aa_corr':'Correlation', 'aa_corr_dot':'correlation','aa_corr_r':'Correlation', 'A':'ALA', 'C': 'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
    'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L': 'LEU', 'N':
    'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER', 'R': 'ARG', 'T': 'THR',
    'W': 'TRP', 'V': 'VAL', 'Y': 'TYR', 'freq_A': 'Fraction of residues', 'freq_C':
    'Fraction of residues', 'freq_E': 'Fraction of residues', 'freq_D': 'Fraction of residues', 'freq_G': 'Fraction of residues',
    'freq_F': 'Fraction of residues', 'freq_I': 'Fraction of residues', 'freq_H': 'Fraction of residues', 'freq_K':
    'Fraction of residues', 'freq_M': 'Fraction of residues', 'freq_L': 'Fraction of residues', 'freq_N': 'Fraction of residues',
    'freq_Q': 'Fraction of residues', 'freq_P': 'Fraction of residues', 'freq_S': 'Fraction of residues', 'freq_R':
    'Fraction of residues', 'freq_T': 'Fraction of residues', 'freq_W': 'Fraction of residues', 'freq_V': 'Fraction of residues',
    'freq_Y': 'Fraction of residues', 'scampi':'TM (%AA)',
    'slider':'Slider','disembl_coils':'DisEMBL Coils (%AA)','disembl_rem465':"DisEMBL REM465 (%AA)",
    'disembl_hotloops':"DisEMBL Hotloops (%AA)", "globpipe":"GlobPlot (%AA)","length":"Length (AA)", "euk":"Eukaryota",
    "pro":"Bacteria",
    "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea'
                     ,"numdiso_long":"Num disorder resdues (long)"
                     ,"numdiso_short":"Num disorder resdues (short)"
                     ,"numseg":"Low complexity residues"
                   }

    # the properties to plot
    properties_x = ["GC_genomic"]
    #properties_x = ["GC", "GC3", "GC_genomic"]

    properties_y = ['top-idp', 'hessa', 'iupred_long', 'iupred_short', 'seg',
                    'ss_alpha', 'ss_beta', 'ss_coil', 'ss_turn','numdiso_long','numdiso_short','numseg',
        'freq_A', 'freq_C', 'freq_D', 'freq_E', 'freq_F', 'freq_G',
        'freq_H', 'freq_I', 'freq_K', 'freq_L', 'freq_M', 'freq_N',
        'freq_P', 'freq_Q', 'freq_R', 'freq_S', 'freq_T', 'freq_V',
        'freq_W', 'freq_Y']
        
        
    
    top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noser = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noserlyspro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0., 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0., 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    top_idp_noserpro = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0., 'Q' : 0.318, 'R' : 0.180, 'S':  0.,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}
        
        
    dic_limits={"hessa":[0.85,1.25],"GC":[20,80],"GC_genomic":[20,80],"taxon_gc":[30,65],"taxon_gc_r":[30,65],"gc_third":[30,65],
                'aa_corr':[0.8,1],'aa_corr_dot':[0.8,1.0],'aa_corr_r':[0.4,1],
                "top-idp":[0,0.15], 'W':[0.006,0.02], 'C':[0.01,0.04],
                'H':[0.01,0.04], 'M':[0.01,0.04], 'Y':[0.01,0.06],
                'Q':[0.02,0.05], 'P':[0.02,0.11], 'F':[0.01,0.08],
                'R':[0.02,0.1], 'G':[0,0.12], 'D':[0.02,0.06],
                'A':[0,0.15], 'V':[0.04,0.08], 'N':[0.01,0.08],
                'T':[0.04,0.08], 'E':[0.02,0.08], 'I':[0,0.12],
                'K':[0.01,0.1], 'S':[0.05,0.11], 'L':[0.06,0.12],
                'freq_W':[0.0,0.025], 'freq_C':[0.00,0.04],
                'freq_H':[0.01,0.035], 'freq_M':[0.01,0.065],
                'freq_Y':[0.01,0.06], 'freq_Q':[0.01,0.07],
                'freq_P':[0.0,0.08], 'freq_F':[0.018,0.08],
                'freq_R':[0.02,0.11], 'freq_G':[0.02,0.12],
                'freq_D':[0.02,0.1], 'freq_A':[0,0.18],
                'freq_V':[0.04,0.11], 'freq_N':[0.01,0.095],
                'freq_T':[0.03,0.08], 'freq_E':[0.02,0.1],
                'freq_I':[0.02,0.14], 'freq_K':[0.01,0.13],
                'freq_S':[0.04,0.11], 'freq_L':[0.075,0.15],
                "iupred_long":[0,40], "tm":[0,12],'idp':[0,0.20],
                "seg":[0,25],"length_aa":[0,720],'iupred_short':[0,40],
                'iupred_cutoff':[0,50],'volatility':[0.7,0.85],'hydropathy':[-1,1],
                "ss_alpha":[-0.05,0.03],"ss_beta":[-0.1,0.02],"ss_coil":[-0.03,0.02],"ss_turn":[-0.11,0.0],
                'slider':[10,80],'disembl_coils':[40,100],'disembl_rem465':[0,80],'disembl_hotloops':[10,90],
                "globpipe":[0,100] } # hardcoded axis limits


    dic_colors = {"Eukaryota":"#9daf6a", "Bacteria":"#b45a5c", "Archaea":"#377eb8"}
    dic_colors_fit = {"Eukaryota":"#bdcf8a", "Bacteria":"#f43a3c", "Archaea":"#579ed8"}
    dic_markers = {"Eukaryota":"o", "Bacteria":"x", "Archaea":"o"}

    
    boxcolr=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue","blue","blue","green"]
    boxcolrnoS=["red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","red","blue","blue","blue","lightblue","lightblue","lightblue","green"]
    




    # Here we start plotting...
    #************************************************************************
    props = ["freq_" + aa for aa in aas_sorted_topidp]
#    props += ["iupred_long", "iupred_short", "top-idp", "topidp-S" ,"topidp-SKP" ,"topidp-SP" , "length"]
    props += ["iupred_long", "iupred_short", "top-idp",'numdiso_long','numdiso_short','numseg',"length_translation"]
    #print (df)
    for prop in props:
        print (prop)
        fig, axes = plt.subplots()

        data = []

        for k in kingdoms:
            if prop in ["length_translation","numdiso_long","numdiso_short","numseg"]:
                d = df.loc[(df.kingdom == k )& (df.taxon_id.isin(good_taxa))][prop]
                num=d.count()+1
                d = d/num
            else:
                length = df.loc[(df.kingdom == k) & (df.taxon_id.isin(good_taxa))]["length_translation"]
                d = df.loc[(df.kingdom == k) & (df.taxon_id.isin(good_taxa))][prop]/length
                #if prop in ["iupred_long", "iupred_short"]:
                #    d = d*100  # ALready multiplied with 100
            #d = [temp/length]
            #print (k,length,d)
                
            data += [d]
            #print (data)
    #axes.violinplot(dataset = data, showmedians = True)
        bp = axes.boxplot(data,patch_artist=True)
    
        #dic_colors = {0:'#E20048', 1:'#5064d0',2:'#00B060',  3:'orange'}
        dic_colors = {0:'#E20048', 1:'#00B060',  2:'#5064d0'}
    
        for i,box in enumerate(bp['boxes']):
            box.set(color = dic_colors[i], linewidth=2)
            box.set(facecolor = dic_colors[i])
        
        
    #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')

        '''
        if prop == "length_translation":
            axes.set_ylim(0,1000)
        elif prop == "top-idp":
            axes.set_ylim(0,0.2)
        elif prop == "topidp-S":
            axes.set_ylim(0,0.2)
        elif prop == "topidp-SP":
            axes.set_ylim(-0.05,0.05)
        elif prop == "topidp-SKP":
            axes.set_ylim(-0.05,0.05)
        else:
            axes.set_ylim(0,50)
        '''
        axes.set_xticks(range(1,len(kingdoms)+1))
        axes.set_xticklabels(kingdoms)
        axes.set_ylabel(ylabels[prop])
        axes.set_title(dic_labels[prop])
        
        #axes.set_ylabel(dic_labels[prop])

    #~ plt.show()
        fig.savefig(figures_dir+"boxplot_kingdoms_" + prop + ".eps")

        plt.close()

    props = ["freq_" + aa for aa in aas_sorted_topidp]
    props += ["top-idp","iupred_long","iupred_short","length_translation"]
    #datasets += uniprot_sets

    #for d in datasets:
    #    print(d,dfs[d])
    
    for k in kingdoms:
        for l in kingdoms:
            if (k < l):
                continue
            for d1 in datasets:
                for d2 in datasets:
                    if (d1==d2 and k==l):
                        continue
                    data=[]
                    rawdata=[]
                    fig, ax = plt.subplots(figsize=(6.,3))
                    for prop in props:
                        length_1 = dfs[d1].loc[(dfs[d1].kingdom == k) & (dfs[d1].taxon_id.isin(good_taxa))]["length_translation"]
                        length_2 = dfs[d2].loc[(dfs[d2].kingdom == l) & (dfs[d2].taxon_id.isin(good_taxa))]["length_translation"]
                        if prop == "length_translation":
                            cat1 = dfs[d1].loc[(dfs[d1].kingdom == k) & (dfs[d1].taxon_id.isin(good_taxa))][prop]
                            cat2 = dfs[d2].loc[(dfs[d2].kingdom == l) & (dfs[d2].taxon_id.isin(good_taxa))][prop]
                        else:
                            cat1 = dfs[d1].loc[(dfs[d1].kingdom == k) & (dfs[d1].taxon_id.isin(good_taxa))][prop]/length_1
                            cat2 = dfs[d2].loc[(dfs[d2].kingdom == l) & (dfs[d2].taxon_id.isin(good_taxa))][prop]/length_2

                        a, b = ttest_ind(cat1, cat2,equal_var=False)
                        c, d  = mannwhitneyu(cat1, cat2,alternative='two-sided')
                        mean1=np.mean(cat1)
                        mean2=np.mean(cat2)
                        sd1=np.sqrt(np.var(cat1))
                        sd2=np.sqrt(np.var(cat2))
                        Z=(mean1-mean2)/math.sqrt(sd1*sd1+sd2*sd2)
                        print ("T-test: ",prop,k,l,d1,d2,mean1,mean2,a,b,c,d)
                        print ("Z-score: ",prop,k,l,d2,d2,mean1,mean2,sd1,sd2,Z)
                        data += [Z]
                        if (prop in ["top-idp","iupred_long","iupred_short"]):
                            mean1=mean1/100
                            mean2=mean2/100
                        rawdata += [mean1-mean2]
                    figname=figures_dir+"/boxplot-Z-test_"+k + "_" + d1 +"_vs_" + l + "_" + d2 + ".eps"
                    ax.bar(range(len(data)),data, color=boxcolr)
                    if k == l:
                        ax.set_ylim([-2.2,3.9])
                    else:
                        ax.set_ylim([-1.4,2.9])
                    #~ ax.set_xticks([0.5 + x for x in range(len(props))])
                    ax.set_xticks([x for x in range(len(props))])
                    ax.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax.set_title("Difference between " + k + " (" + dic_labels[d1] + ") and " + l + " (" +  dic_labels[d2] + ")" )
                    ax.set_ylabel("Z-score")
                    ax.yaxis.grid(True)
                    ax.xaxis.grid(True)
                    fig.savefig(figname, bbox_inches = "tight")
                    plt.close()

                    fig2, ax2 = plt.subplots(figsize=(6.,3))
                    # Let's test the raw data..
                    figname=figures_dir+"/boxplot-raw-test_"+k + "_" + d1+"_vs_" + l + "_" + d2 + ".eps"
                    ax2.bar(range(len(rawdata)),rawdata, color=boxcolr)
                    #ax2.set_ylim([-0.03,0.03])
                    #~ ax2.set_xticks([0.5 + x for x in range(len(props))])
                    ax2.set_xticks([x for x in range(len(props))])
                    ax2.set_xticklabels([dic_labels[x] for x in props], rotation=90,fontsize=8)
                    ax2.set_title("Difference between " + k + " (" + d1 + ") and " + l + " (" + d2 + ")" )
                    ax2.set_ylabel("Difference")
                    ax2.yaxis.grid()
                    ax2.xaxis.grid()
                    fig2.savefig(figname, bbox_inches = "tight")
                    plt.close()
                    
                
    # Print latex table
    '''
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    for prop in props:
        string=dic_labels[prop]
        for kingdom in ["Eukaryota", "Bacteria"]:
            for ty in datasets:
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                string += " & " + str(round(100*(np.mean(value))/(np.mean(length)),2)) + "\%" 
        string += "\\\\ "
        print (string)
    '''        

    props = ["freq_" + aa for aa in aas_sorted_topidp]
    for prop in props:
        string=dic_labels[prop]
        for kingdom in ["Eukaryota", "Bacteria"]:
            for ty in datasets:
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]/length
                string += " & " + str(round(100*(np.mean(value)),1)) + "\%" 
        string += "\\\\"
        print (string)
            

    #************************************************************************

    ##########################################################################################################################
    # for each amino acid, and separately for eukarya and bacteria, plot its frequency versus the taxonomic GC% for 4 curves:
    # 1- average freq
    # 2- a linear fit of the data
    # 3- the frequency as it should be if only dependent on GC
    # 4- a model function (w * prob_tot + k) with estimated parameters W and K
    # note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

    # chose which dataset to use  for the following plots



    
    #good_taxa = set(df.taxon_id.unique())
    #print ("TEST all:",df.taxon_id.unique())
    #print ("TEST good:",good_taxa)
    '''
    res = {}
    for kingdom in [b"Bacteria", b"Eukaryota"]:
        #for kingdom in kingdoms:
        res[kingdom] = {}
    
        for aa in ['L', 'D', 'V', 'C', 'E', 'T', 'H', 'S',
            'M', 'W', 'Y', 'F', 'Q', 'P', 'G', 'N', 'R', 'I',
            'K', 'A']:
            
            global aminoacid
            aminoacid = aa
            
            # get the distribution of the frequency in bacteria
            d = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))][["freq_"+aa,"GC_genomic"]].sort_values("GC_genomic").dropna(subset=["GC_genomic"])
            #            d = df.loc[(df.kingdom == kingdom) ][["freq_"+aa,"GC_genomic"]].sort_values("GC_genomic").dropna(subset=["GC_genomic"])
            
            xdata = d["GC_genomic"]/100.0
            ydata = d["freq_"+aa]


            fig, ax = plt.subplots()
            
            # plot the data
            ax.scatter(xdata,ydata,color="lightgrey",edgecolor="None",alpha=0.3,label = "Data")
            
            
            # get an estimate of the error given only the average AA frequence
            ydata_mean = ydata.mean()
            ydata_std = ydata.std()
            avg_ydata = [ydata_mean for _ in ydata]
            ax.plot(xdata,avg_ydata,color="blue",linestyle="--",label = "Average")
            
            
            # create a first estimate of the fitting function and plot it
            initial_w = 1
            initial_k = 0
            estimated_ydata = my_func(xdata,ydata,initial_w,initial_k)
            ax.plot(xdata,estimated_ydata,color="darkred",linestyle="--",label = "GC Simple")
            
            
            # do a curve fit to estimate the parameters
            popt, pcov = curve_fit(my_func, xdata, ydata )
            _, est_w, est_k = popt
            # limit the parameters
            est_w = clamp(est_w, 0, 1)
            est_k = clamp(est_k, 0, ydata.mean())
            optimized_ydata = my_func(xdata, ydata, est_w, est_k)
            ax.plot(xdata, optimized_ydata, color="orange",lw=2, label='Function')
            
            
            # calculate and plot the fitting lines
            fit = np.polyfit(xdata,ydata,2)
            fit_fn = np.poly1d(fit) 
            fitted_ydata = fit_fn(xdata)
            plt.plot(xdata, fitted_ydata, lw=2,color="green",label="Linear Fit")
            
            
            res[kingdom][aa] = est_w
            
            
            # calculate and displays errors
            error_avg = get_error(ydata, avg_ydata) / ydata_std
            error_gc = get_error(ydata, estimated_ydata) / ydata_std
            error_func = get_error(ydata, optimized_ydata) / ydata_std
            error_fit = get_error(ydata, fitted_ydata) / ydata_std
            
            #text_str = "Avg Error = " + str(round(error_avg,4)) + "\n"
            #text_str += "GC Error = " + str(round(error_gc,4)) + "\n"
            #text_str += "Function Error = " + str(round(error_func/ydata_std,4)) + "\n"
            #text_str += "Fit Error = " + str(round(error_fit/ydata_std,4)) + "\n"
            text_str = "Error = " + str(round(get_error(fitted_ydata, optimized_ydata) / ydata_std,4)) + "\n"
            
            text_str += "W = " + str(round(est_w,5)) + "\n"
            text_str += "K = " + str(round(est_k,5)) 

            bb = ax.text(ax.get_xlim()[0], ax.get_ylim()[1], text_str, fontsize=12,
                    ha="left", va="top",
                    bbox=dict(boxstyle="square,pad=0",
                            ec=(0.0, 0.0, 0.0,1),
                            fc=(1., 1, 1,1),),    
                        )
            
            
            ax.set_xlabel("GC%")
            ax.set_ylabel("Frequency")
            ax.set_title(dic_labels[aa] +  " - " + dic_labels[kingdom])
            ax.legend(loc="lower left")
            
            
            fig_title =  figures_dir + "aafreq_" + aa + "_" + kingdom  + ".eps"
            
            print(fig_title)
            fig.savefig(fig_title)
            #fig.show()
            plt.close()
            
        #############################
    '''
    # examples

    
    for k in kingdoms:
            for j in kingdoms:
                if (j<k):
                    continue
                for d in datasets:
                        for e in datasets:
                            bar_plot_s1_vs_s2(e,d, j, k,False,False)
                            bar_plot_s1_vs_s2(e,d, j, k,True,False)



    #######  HEATMAP
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    data = []
    y_labels = []
    for kingdom in ["Eukaryota", "Bacteria"]:
        for ty in datasets:
            if ty != "full": # Just for checking we include fill as well
                d = []
                for prop in props:
                    length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"]
                    value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop]/length
                    d += [np.mean(value)]
                    
                    #print (kingdom, ty,prop, value)
                    
                y_labels += ["  " + dic_labels[kingdom] + "\n  " + dic_labels[ty]]
                
                data += [d]
                

        

    sns.set()
    sns.set(font_scale=1.4)
    
    np.savetxt("freq.csv", data, delimiter=' & ', fmt='%2.2e', newline=' \\\\\n')


    sns.clustermap(data, col_cluster=False,yticklabels=y_labels, xticklabels=aas_sorted_topidp,cmap="Purples",annot=False, annot_kws={"size": 3},metric='correlation')

    plt.savefig(figures_dir + "aa_freq_heatmap.eps",bbox_inches="tight")

    #################################################

    
    ######################################################

    # for a given dataset (ex. "dataset") plot two properties against each other


        
    dic_colors= {"Eukaryota":'#00B060', "Bacteria":'#E20048',"Archaea":'#5064d0'}
    for prop_1 in properties_x:
        for prop_2 in properties_y:
            #print ("TEST",prop_1,prop_2)
            fig, ax = plt.subplots(figsize=(6,6))
        
            # plot the values as a scatter plot
            prop_1_values = {}
            prop_2_values = {}
            for kingdom in ['Eukaryota','Bacteria','Archaea']:
                

                good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)
                
                temp_df = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))][list(set([prop_1,prop_2,"iupred_short"]))].dropna()
                
                prop_1_values[kingdom] = temp_df[prop_1].values
                prop_2_values[kingdom] = temp_df[prop_2].values
                
                ax.scatter(prop_1_values[kingdom],prop_2_values[kingdom], edgecolor="none", facecolor=dic_colors[kingdom],alpha = 1,s=50,label=dic_labels[kingdom])
                #sct = ax.scatter(prop_1_values[kingdom],prop_2_values[kingdom],marker=dic_markers[kingdom], c=colors_list, edgecolor="none",alpha = 1,s=70,label=dic_labels[kingdom])
                
                # calculate and plot the fitting lines
                fit = np.polyfit(list(prop_1_values[kingdom]),list(prop_2_values[kingdom]),1)
                fit_fn = np.poly1d(fit) 
                plt.plot(prop_1_values[kingdom], fit_fn(prop_1_values[kingdom]), '--k',lw=2,color=dic_colors_fit[kingdom])

                #slope, intercept, r_value, p_value, std_err = linregress(prop_1_values, prop_2_values)
                #print(slope, intercept, r_value, p_value, std_err)
                

            # plot the random running average
            if prop_1 in df_genes_random.columns and prop_2 in df_genes_random.columns:
                prop_1_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_1].astype(float)),500)
                prop_2_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_2].astype(float)),500)
                ax.plot(prop_1_random,prop_2_random,color="black",label="Random")
                
            # plot the theoretical line
            if prop_2.split("_")[0] == "freq" and prop_1 in ["GC","GC3","GC_genomic"]:
                aa = prop_2.split("_")[1]
                p = []
                for gc in np.arange(0.1,0.9,0.001):
                    p += [get_aa_freq(aa, gc)]
                
                ax.plot(np.arange(10,90,0.1),p,color="blue",lw=2,label="Theoretical")


            # set the limits
            if prop_1 in dic_limits:
                ax.set_xlim(dic_limits[prop_1])
                
            if prop_2 in dic_limits:
                ax.set_ylim(dic_limits[prop_2])
                
                
            # set the labels
            ax.set_xlabel(dic_labels.get(prop_1,prop_1))
            ax.set_ylabel(dic_labels.get(prop_2,prop_2))
            
            # plot legend
            ax.legend(loc="upper right")
            
            # ranksums tests
            ranksum_string = ""
            ranksum_string += "Eukarya vs Bacteria = " + fmt_scientific(stats.ranksums(prop_2_values["Eukaryota"], prop_2_values["Bacteria"])[1]) + "\n"
            ranksum_string += "Eukarya vs Archaea = " + fmt_scientific(stats.ranksums(prop_2_values["Eukaryota"], prop_2_values["Archaea"])[1]) + "\n"
            ranksum_string += "Bacteria vs Archaea = " + fmt_scientific(stats.ranksums(prop_2_values["Bacteria"], prop_2_values["Archaea"])[1]) + "\n" + "\n"
            
            # Pearson correlation coefficient
            ranksum_string += "Eukarya R = " + str(round(stats.pearsonr(prop_1_values["Eukaryota"], prop_2_values["Eukaryota"])[0],2)) + "\n"
            ranksum_string += "Bacteria R = " + str(round(stats.pearsonr(prop_1_values["Bacteria"], prop_2_values["Bacteria"])[0],2)) + "\n"
            ranksum_string += "Archaea R = " + str(round(stats.pearsonr(prop_1_values["Archaea"], prop_2_values["Archaea"])[0],2))
            
            
            # calculate ABS difference between Euk-theoretical and Prok-theoretical
            if prop_2.split("_")[0] == "freq" and prop_1 in ["GC","GC3","GC_genomic"]:
                
                diff_from_theo = {}
                
                aa = prop_2.split("_")[1]
                for kingdom in ['Eukaryota','Bacteria']:
                    p = []
                    for vx,vy in zip (prop_1_values[kingdom], prop_2_values[kingdom]):
                        p += [abs(vy-get_aa_freq(aa, vx/100.0))]
                        
                    ranksum_string += "abs(" + kingdom + " - theoretical) = " + str(round(np.mean(p),4))+ "\n"
                    
                    
            bb = ax.text(ax.get_xlim()[0], ax.get_ylim()[1], ranksum_string, fontsize=12,
                ha="left", va="top",
                bbox=dict(boxstyle="square,pad=0",
                        ec=(0.0, 0.0, 0.0,1),
                        fc=(1., 1, 1,1),),    
                    )
            
        
            
            figname = figures_dir + prop_2 + "_vs_" + prop_1 + "_pfam.eps"
            print(figname)
            fig.savefig(figname)
            #fig.show()
            
            ##########################################################################################################################
            
            
            
            
        ##########################################################################################################################
        
    # plot a series of properties as boxplots, with each box corresplonding to a dataset-kingdom combination
    
    # the properties to plot
    props = ["iupred_long", "iupred_short", "top-idp","seg","length_translation",'numdiso_long','numdiso_short','numseg']
    props += ["freq_A","freq_C","freq_D","freq_E","freq_F","freq_G","freq_H","freq_I","freq_K",
                  "freq_L","freq_M","freq_N", "freq_P","freq_Q","freq_R","freq_S", "freq_T",
                  "freq_V","freq_W","freq_Y"]



    dic_boxplot_limits = {"iupred_long":[0,50], "iupred_short":[0,50], 
                      "top-idp":[0,0.15],'length_translation':[0,800],
                      'hessa':[0.85,1.2], 'seg':[0,25],"scampi":[0.02,0.1],
                    "freq_S":[0.03,0.1],"freq_P":[0.01,0.08], 
                      "freq_I":[0.02,0.11],"freq_T":[0.03,0.08],
                              "freq_A":[0.00,0.18],
                              "freq_C":[0.0,0.06],
                              "freq_D":[0.02,0.1],
                              "freq_E":[0.01,0.12],
                              "freq_F":[0.00,0.1],
                              "freq_G":[0.03,0.15],
                              "freq_H":[0.00,0.06],
                              "freq_K":[0.00,0.12],
                              "freq_L":[0.05,0.15],
                              "freq_M":[0.01,0.05],
                              "freq_N":[0.00,0.1],
                              "freq_Q":[0.01,0.1],
                              "freq_R":[0.01,0.1],
                              "freq_V":[0.03,0.12],
                              "freq_W":[0.0,0.04],
                              "freq_Y":[0.00,0.07],
                              }

    




    for prop in props:
    
        all_data = []
        labels = []
        for d in datasets:
            for k in kingdoms:

                data = dfs[d].loc[dfs[d].kingdom == k][prop].dropna()  
                length = dfs[d].loc[dfs[d].kingdom == k]["length_translation"].dropna()  
                all_data += [data/length]
                
                label = k + "_" + d
                labels += [label]
                
        fig, ax = plt.subplots(figsize=(4,3))
        
        plt.xticks(rotation=90)
        bp = ax.boxplot(all_data,labels=labels,patch_artist=True)
        
        
        for i,box in enumerate(bp['boxes']):
            if i%2 == 0:
                box.set(color='#93002F', linewidth=2)
                box.set(facecolor = '#E20048' )
            else:
                box.set(color='#00733E', linewidth=2)
                box.set(facecolor = '#00B060' )

        #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')
        
        ax.set_title(dic_labels[prop])
        #ax.set_ylim(dic_boxplot_limits[prop])
        
        
        figname = figures_dir + "boxplot_" + prop + ".eps"
        print(figname)
        fig.savefig(figname,bbox_inches = "tight")
        #fig.show()

 

    
    # plot a series of properties as boxplots, with each box corresplonding to phylum
    
    # the properties to plot props = ["iupred_long", "iupred_short", 
    #"top-idp",'length_translation', 'hessa',
    #'seg','GC_genomic','numdiso_long','numdiso_short','numseg']
    
    props += ["freq_A","freq_C","freq_D","freq_E","freq_F","freq_G","freq_H","freq_I","freq_K",
                  "freq_L","freq_M","freq_N", "freq_P","freq_Q","freq_R","freq_S", "freq_T",
                  "freq_V","freq_W","freq_Y"]



    phyla_euk = dfs["full"].loc[dfs["full"].kingdom == "Eukaryota"]["phylum"].unique()
    phyla_proks = dfs["full"].loc[dfs["full"].kingdom == "Bacteria"]["phylum"].unique()
    phyla_arch = dfs["full"].loc[dfs["full"].kingdom == "Archaea"]["phylum"].unique()
    #~ all_phyla = list(phyla_proks) + list(phyla_euk)  + list(phyla_arch)
    all_phyla = list(phyla_proks) + list(phyla_euk)
    
    
    
    for prop in props:    
        data = []
        labels = []
        for phylum in all_phyla:
            dd = dfs["full"].loc[dfs["full"].phylum == phylum][prop].dropna()
            if len(dd) > 10:
                length = dfs["full"].loc[dfs["full"].phylum == phylum]["length_translation"].dropna()
                data += [dd/length]
                labels += [phylum]
        
        fig, ax = plt.subplots(figsize=(6,3))
        ax.set_ylabel('Fraction')
        plt.xticks(rotation=90)
        bp = ax.boxplot(data,labels=labels,patch_artist=True)
        
        dic_colors= {"Eukaryota":'#00B060', "Bacteria":'#E20048'}
        
      
        for l,box in zip(labels,bp['boxes']):
            if l in set(phyla_euk):
                box.set(color='#00B060', linewidth=2)
                box.set(facecolor = '#00B060' )
            elif l in set(phyla_proks):
                box.set(color='#E20048', linewidth=2)
                box.set(facecolor = '#E20048' )
            else:
                box.set(color='orange', linewidth=2)
                box.set(facecolor = 'orange' )

        #~ pylab.setp(bp['boxes'],color='black',lw=2)
        plt.setp(bp['whiskers'], color='black')
        plt.setp(bp['medians'], color='black')
        plt.setp(bp['fliers'], color='black', marker='+')
        
    
        ax.set_title(dic_labels[prop])
        
        
        figname = figures_dir + "boxplots_by_phylum_" + prop + ".eps"
        print(figname)
        fig.savefig(figname,bbox_inches = "tight")
        #fig.show()
        plt.close()

    



    # Print latex table
    props = ["freq_" + aa for aa in aas_sorted_topidp]
    for prop in props:
        string=dic_labels[prop]
        for kingdom in ["Eukaryota", "Bacteria"]:
            for ty in datasets:
                value = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))][prop].mean()
                length = dfs[ty].loc[(dfs[ty].kingdom == kingdom) & (dfs[ty].taxon_id.isin(good_taxa))]["length_translation"].mean()
                string += " & " + str(round(100*(value/length),2)) + "\%" 
        string += "\\ "
        print (string)
            
