#!/usr/bin/env python
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
from Bio.PDB import *
import os.path
from Bio import motifs
import sys


class PDBSelect(Select):
    def accept_residue(self, residue):
        if residue.get_name()=='GLY':
            return True
        else:
            return False

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

def parse_annotation(filename):

    tax_id = int(filename.split("_")[1].split(".")[0])
    print (tax_id)

    df = pd.read_csv(filename)
    n_proteins = len(df.query_id)

    df_mean = df.mean()
    
    columns = ["length", "top-idp", "iupred_long", "iupred_short","seg","ss_alpha", "ss_beta", "ss_coil", "ss_turn","hessa"]
    columns += ["freq_" + aa for aa in aas]
    columns += ["pfamA_acc"]
    
    ret_dic = {}    
    ret_dic["taxon_id"] = tax_id
    ret_dic["phylum"] = taxid2phylum.get(tax_id,pd.np.nan)
    ret_dic["kingdom"] = taxid2kingdom.get(tax_id,pd.np.nan)
    ret_dic["GC"] = taxid2gc.get(tax_id,pd.np.nan)
    ret_dic["count_protein"] = n_proteins

    for c in columns:
        ret_dic[c] = df_mean[c]
    
    #gcs = df_reference.loc[df_reference["TaxID"] == tax_id]["GC%"].astype(float)
    #ret_dic["GC"] = pd.np.mean(list(gcs))

    return ret_dic

if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']

    data_dir = "../data/"
    pdb_dir = "../data/pdb/"
    results_dir = "../data/"
    input_dir = "../data/pfam/"
    pfam_dir = "../data/pfam/"
    cifparser = MMCIFParser()
    pdbparser = PDBParser()
    
    # This takes too long
    '''
    print "reading taxonomy tab"
    df_taxonomy = pd.read_csv(data_dir + "taxonomy.tab", sep="\t")
    print "Set taxon index"
    memo2taxid = df_taxonomy.set_index("Mnemonic").to_dict()["Taxon"]
    print "Get phylym"
    df_taxonomy["Phylum"] = df_taxonomy.Lineage.apply(get_phylum)
    print "Setting phylum index"
    taxid2phylum = df_taxonomy.set_index("Taxon").to_dict()["Phylum"]
    
    df_taxonomy["Kingdom"] = df_taxonomy.Lineage.apply(get_kingdom)
    print "Setting phylum index"
    taxid2kingdom = df_taxonomy.set_index("Taxon").to_dict()["Kingdom"]
    '''
    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2gc={}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)
    '''
    # get PDB daa
    pdb2sprot={}
    sprot2pdb={}
    with open(data_dir+'pdbtosp.txt') as fp:
        for line in fp:
            if re.match('\d[A-Z0-9][A-Z0-9][A-Z0-9]\s.*[A-Z0-9]_[A-Z0-9]',line):
                s=line.split()
                pdb=s[0]
                s=line[28:38]
                sprot=s.replace('\s','')
                pdb2sprot[pdb]=sprot
                sprot2pdb[sprot]=pdb
    '''
    file_list = []
    for f in os.listdir(input_dir):
        if f.endswith(".sto"):
            file_list += [input_dir + f]
    #for f in sys.argv[1:]:
    #    file_list += [f]
    cutoff=10
    alignment={}
    for f in file_list:
        print (f)
        #bact=MultipleSeqAlignment()
        #euk=MultipleSeqAlignment()
        bact=Bio.Align.MultipleSeqAlignment([])
        arch=Bio.Align.MultipleSeqAlignment([])
        euk=Bio.Align.MultipleSeqAlignment([])
        s=re.search('PF\d\d\d\d\d',f)
        pfam=s.group(0)
        if os.path.isfile(pfam_dir+pfam+"_arch.fa"):
            continue
        alignment = AlignIO.read(f, "stockholm")
        bact_seq=[]
        arch_seq=[]
        euk_seq=[]
        for record in alignment:
            #print (record.name)
            mtemp=re.split("[_]",record.name)
            mnemonic=mtemp[1]
            sprot=record.name
            try:
                pdb=sprot2pdb[sprot]
                #print ("PDB:",pfam,sprot,pdb)
            except:
                pdb=''
            tax_id=memo2taxid.get(mnemonic,pd.np.nan)
            #phylum = taxid2phylum.get(tax_id,pd.np.nan)
            kingdom = taxid2kingdom.get(tax_id,pd.np.nan)
            gc = taxid2gc.get(tax_id,pd.np.nan)
            length=len(record.seq)
            if (kingdom == 'Bacteria' or kingdom == 'Eukaryota'  or kingdom == 'Archaea' ):
                if (kingdom == 'Bacteria'):
                    #print ("Bacteria",record)
                    bact.append(record)
                    bact_seq+=[record.seq]
                elif(kingdom == 'Eukaryota'):
                    #print ("Eukaryot",record)
                    euk.append(record)
                    euk_seq+=[record.seq]
                elif(kingdom == 'Archaea'):
                    #print ("Eukaryot",record)
                    arch.append(record)
                    arch_seq+=[record.seq]
                '''
                for pdb in record.dbxrefs:
                    s=re.split(';',pdb)
                    #print (pfam,record.name,s)
                    p=re.split('\s',s[1])
                    #print (p)
                    pdbcode=p[1].lower()
                    pdbchain=p[2].upper()
                    start,stop=re.split('-',s[2])
                    start=start.replace('\s','')
                    stop=stop.replace('\s','')
                    first=int(start)
                    last=int(stop)
                    fname=pdb_dir+pdbcode+".cif"
                    temp=pdb_dir+pfam+"-"+kingdom+"-"+pdbcode+"_"+pdbchain+"-"+start+"-"+stop+".pdb"
                    pdbfname=temp.replace(' ','')
                    if (not os.path.isfile(fname)):
                        pdbl = PDBList()
                        pdbl.retrieve_pdb_file(pdbcode,pdir=pdb_dir)
                    if ( (not os.path.isfile(pdbfname)) and (os.path.isfile(fname))):
                        structure = cifparser.get_structure("PDB", fname)
                        # Now write out the selected part as 
                        extract(structure, pdbchain, first, last, pdbfname)
                    #sys.exit()
                '''
        AlignIO.write(bact, pfam_dir+pfam+"_bac.fa", "fasta")
        AlignIO.write(arch, pfam_dir+pfam+"_arch.fa", "fasta")
        AlignIO.write(euk, pfam_dir+pfam+"_euk.fa", "fasta")

        '''
        # Some statistics
        minnum=10
        tiny=1.e-20
        sumser_bac=tiny
        sumser_euk=tiny
        sumall_bac=tiny
        sumall_euk=tiny
        for i in range (0,length):
            #print(bact[:, i])
            #print(euk[:, i])
            x = ProteinAnalysis(bact[:, i])
            aafreq_bac=x.count_amino_acids()
            sum_bac=sum(aafreq_bac.values())
            y = ProteinAnalysis(euk[:, i])
            aafreq_euk=y.count_amino_acids()
            sum_euk=sum(aafreq_euk.values())
            # Decide on some cutoffs
            ser_bac=aafreq_bac["S"]
            ser_euk=aafreq_euk["S"]
            sumall_bac+=sum_bac
            sumall_euk+=sum_euk
            sumser_bac+=ser_bac
            sumser_euk+=ser_euk
            if (sum_bac>minnum and sum_euk>minnum):
                if (ser_bac/(sum_bac+tiny)>0.5 or ser_euk/(sum_euk+tiny)>0.5):
                    logodds=math.log10((tiny+ser_bac/(sum_bac))/(tiny+ser_euk/(sum_euk)))
                    #print (pfam,i,logodds,aafreq_bac["S"],sum_bac,aafreq_euk["S"],sum_euk)
            
        # The motifs package does not handle "-"
        #print ("TEST",bact_seq,euk_seq)
        #m_bac= motifs.create(bact_seq)
        #m_euk= motifs.create(euk_seq)
        #m_bac.weblogo(data_dir+pfam+"_bac.png")
        #m_euk.weblogo(data_dir+pfam+"_euk.png")
        # Ok what to do next..
        # get AA frequencies..
        #print (pfam,i,aafreq_bac,sum(aafreq_bac.values()),aafreq_euk,sum(aafreq_euk.values()))
        #print ("Summary:",pfam,sumser_bac/sumall_bac,sumser_euk/sumall_euk,math.log10((sumser_bac/sumall_bac)/(sumser_euk/sumall_euk)))
        
        
        print ("\\begin{figure*}[tb!]")
        print ("\\begin{tabular}{c}")
        print ("\subfloat[][]{\includegraphics[width = 15cm]{figures/"+pfam+"_bac.eps}}\\\\")
        print ("\subfloat[][]{\includegraphics[width = 15cm]{figures/"+pfam+"_euk.eps}}\\\\")
        print ("\end{tabular}")
        print ("\caption{Sequence LOGO of "+pfam+" (Bacteria top) ~\cite{weblogo} ")
        print ("Serine Frequency bacteria:" + str(round(100*sumser_bac/sumall_bac,1))+"%")
        print ("Serine Frequency eukaryota:" + str(round(100*sumser_euk/sumall_euk,1))+"%")
        print ("}")
        print ("\label{fig:logo"+pfam+"}")
        print ("\end{figure*}")

        

        if (len(bact)>cutoff and len(euk)>cutoff):a
            for record in bact:
                x = ProteinAnalysis(str(record.seq))
                aafreq_bac+=x.count_amino_acids()
            for record in euk: 
                x = ProteinAnalysis(str(record.seq))
                aafreq_euk=+x.count_amino_acids()
            print (pfam,aafreq_bac,aafreq_euk)

        #sys.exit()
        '''
