#!/usr/bin/env python3
import pandas as pd
import os
from Bio.Seq import Seq
from Bio import SeqIO
from Bio import AlignIO
import Bio.Align
import re
from Bio.SeqUtils.ProtParam import ProteinAnalysis
import math
from Bio.PDB import *
import os.path
from Bio import motifs
import sys
from random import *
import numpy

class PDBSelect(Select):
    def accept_residue(self, residue):
        if residue.get_name()=='GLY':
            return True
        else:
            return False

def count_subst(a,b):
    for i in len(a):
        i=j
    return i

def get_phylum(s):
    try:
        return s.split(";")[1]
    except:
        return None    

def get_kingdom(s):
    try:
        return s.split(";")[0]
    except:
        return None    

def parse_annotation(filename):

    tax_id = int(filename.split("_")[1].split(".")[0])
    print (tax_id)

    df = pd.read_csv(filename)
    n_proteins = len(df.query_id)

    df_mean = df.mean()
    
    columns = ["length", "top-idp", "iupred_long", "iupred_short","seg","ss_alpha", "ss_beta", "ss_coil", "ss_turn","hessa"]
    columns += ["freq_" + aa for aa in aas]
    columns += ["pfamA_acc"]
    
    ret_dic = {}    
    ret_dic["taxon_id"] = tax_id
    ret_dic["phylum"] = taxid2phylum.get(tax_id,pd.np.nan)
    ret_dic["kingdom"] = taxid2kingdom.get(tax_id,pd.np.nan)
    ret_dic["GC"] = taxid2gc.get(tax_id,pd.np.nan)
    ret_dic["count_protein"] = n_proteins

    for c in columns:
        ret_dic[c] = df_mean[c]
    
    #gcs = df_reference.loc[df_reference["TaxID"] == tax_id]["GC%"].astype(float)
    #ret_dic["GC"] = pd.np.mean(list(gcs))

    return ret_dic

if __name__ == '__main__':
    aas = ['A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
    aaindex = {'A':0, 'C':1, 'D':2, 'E':3, 'F':4, 'G':5,
                   'H':6, 'I':7, 'K':8, 'L':9, 'M':10, 'N':11, 'P':12, 'Q':13,
                   'R':14, 'S':15, 'T':16, 'V':17, 'W':18, 'Y':19}

    data_dir = "../data/"
    pdb_dir = "../data/"
    results_dir = "../data/"
    input_dir = "../data/pfam/"
    output_dir = "../data/substmat/"
    cifparser = MMCIFParser()
    pdbparser = PDBParser()
    
    # This takes too long
    codes={"V":"Viruses","E":"Eukaryota","B":"Bacteria","A":"Archaea","O":"Other","X":"Unknown"}
    taxid2kingdom={}
    memo2taxid={}
    with open(data_dir+'speclist.txt') as fp:
        for line in fp:
            if re.match('.*:\sN=',line):
                s=line.split()
                taxid=s[2].replace(':','')
                mnemonic=s[0]
                kingdom=codes[s[1]]
                memo2taxid[mnemonic]=taxid
                taxid2kingdom[taxid]=kingdom
                #print (line, taxid,mnemonic,kingdom)
    file_list = []
    for f in os.listdir(input_dir):
        if f.endswith(".sto"):
            file_list += [input_dir + f]
    #for f in sys.argv[1:]:
    #    file_list += [f]
    cutoff=10
    alignment={}
    subst_bb=numpy.zeros((20, 20))
    subst_ee=numpy.zeros((20, 20))
    subst_be=numpy.zeros((20, 20))
    num_e=numpy.zeros(20)
    num_b=numpy.zeros(20)
    num_bb=numpy.zeros(20)
    num_ee=numpy.zeros(20)
    #num_be=numpy.zeros(20)
    maxseq=1000
    for f in file_list:
        #print (f)
        #bact=MultipleSeqAlignment()
        #euk=MultipleSeqAlignment()
        bact=Bio.Align.MultipleSeqAlignment([])
        euk=Bio.Align.MultipleSeqAlignment([])
        s=re.search('PF\d\d\d\d\d',f)
        pfam=s.group(0)
        if (os.path.isfile(output_dir+pfam+"-subst_be.npy")):
            continue
        print ("Reading alignments",pfam)
        alignment = AlignIO.read(f, "stockholm")
        bact_seq=[]
        euk_seq=[]
        print ("Classifying alignments",pfam)
        numbac=0
        numeuk=0
        j=0
        for record in alignment:
            #print (j,numbac,numeuk)
            j+=1
            mtemp=re.split("[_]",record.name)
            mnemonic=mtemp[1]
            sprot=record.name
            tax_id=memo2taxid.get(mnemonic,pd.np.nan)
            #phylum = taxid2phylum.get(tax_id,pd.np.nan)
            kingdom = taxid2kingdom.get(tax_id,pd.np.nan)
            #length=len(record.seq)
            if (kingdom == 'Bacteria' and numbac<=maxseq):
                bact.append(record)
                bact_seq+=[record.seq]
                numbac+=1
            elif(kingdom == 'Eukaryota' and numeuk<=maxseq):
                euk.append(record)
                euk_seq+=[record.seq]
                numeuk+=1
            if ((numbac>maxseq and numeuk>maxseq) or j>10000):
                break
        #AlignIO.write(bact, input_dir+pfam+"_bac.fa", "fasta")
        #AlignIO.write(euk, input_dir+pfam+"_euk.fa", "fasta")

        # Select 100 random pair of each group (bac-bac, euk-euk and bac-euk)
        print ("Read sequences",numbac,numeuk)
        if (numeuk<10 or numbac<10  ):
            continue
        if ( float(numeuk/numbac)<1.e-3 or float(numbac/numeuk)<1.e-3  ):
            continue
        print ("starting randomization")
        for i in range(1,100):
            baclen=len(bact)
            euklen=len(euk)
            # Bac-bac
            x=randint(0,baclen-1)
            y=randint(0,baclen-1)
            A=bact[x].seq
            B=bact[y].seq
            A=A.upper()
            B=B.upper()
            #print (x,y,A,B)
            #sys.exit()
            for j in range(0,len(A)):
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', A[j]) and re.match('[ACDEFGHIKLMNPQRSTVWY]', B[j])):
                    num_bb[aaindex[A[j]]]+=1
                    num_bb[aaindex[B[j]]]+=1
                    subst_bb[aaindex[A[j]],aaindex[B[j]]]+=1
                    subst_bb[aaindex[B[j]],aaindex[A[j]]]+=1
            # Euk-EUk
            x=randint(0,euklen-1)
            y=randint(0,euklen-1)
            A=euk[x].seq
            B=euk[y].seq
            #print (A,B)
            A=A.upper()
            B=B.upper()
            #sys.exit()
            for j in range(0,len(A)):
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', A[j]) and re.match('[ACDEFGHIKLMNPQRSTVWY]', B[j])):
                    num_ee[aaindex[A[j]]]+=1
                    num_ee[aaindex[B[j]]]+=1
                    subst_ee[aaindex[A[j]],aaindex[B[j]]]+=1
                    subst_ee[aaindex[B[j]],aaindex[A[j]]]+=1

            # Bac-euk
            x=randint(0,baclen-1)
            y=randint(0,euklen-1)
            A=bact[x].seq
            B=euk[y].seq
            #print (A,B)
            A=A.upper()
            B=B.upper()
            #sys.exit()
            for j in  range(0,len(A)):
                if (re.match('[ACDEFGHIKLMNPQRSTVWY]', A[j]) and re.match('[ACDEFGHIKLMNPQRSTVWY]', B[j])):
                    num_b[aaindex[A[j]]]+=1
                    num_e[aaindex[B[j]]]+=1
                    subst_be[aaindex[A[j]],aaindex[B[j]]]+=1

        #print ("BB",num_bb,subst_bb)
        #print ("EE",num_ee,subst_ee)
        #print ("BE",num_be,subst_be)

        numpy.save(output_dir+pfam+"-num_b.npy",num_b)
        numpy.save(output_dir+pfam+"-num_e.npy",num_e)
        numpy.save(output_dir+pfam+"-num_ee.npy",num_ee)
        numpy.save(output_dir+pfam+"-num_bb.npy",num_bb)
        numpy.save(output_dir+pfam+"-subst_bb.npy",subst_bb)
        numpy.save(output_dir+pfam+"-subst_ee.npy",subst_ee)
        numpy.save(output_dir+pfam+"-subst_be.npy",subst_be)
        #sys.exit()
