#!/usr/bin/env python3
from __future__ import print_function



def my_func(x,a,b):
    global aminoacid,ydata_mean
    codons = aa2codons[aminoacid]
    stopcodons = aa2codons['_']
    res = []
    mean=ydata_mean
    prob_avg = len(codons)/61.
    for gc in x:
        stop_tot = 0.
        for codon in stopcodons:
            prob_codon = []
            for N in codon:
                #print (gc)
                prob = float(gc)/2
                if N in {"A","T","U"}:
                    prob = 0.5-(float(gc)/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            stop_tot += functools.reduce(mul,prob_codon)
        prob_tot = 0.
        for codon in codons:
            prob_codon = []
            for N in codon:
                prob = float(gc)/2
                if N in {"A","T","U"}:
                    prob = 0.5-(float(gc)/2)
                prob_codon += [prob]
            #print(codon, prob_codon,functools.reduce(mul,prob_codon))
            prob_tot += functools.reduce(mul,prob_codon)
        #prob_tot = sum(prob_tot)
        prob_tot=prob_tot*(1/(1-stop_tot))  # To compensate for stop codons.
        res += [a * (prob_tot - prob_avg) + prob_avg + b]  # This just means  [w * prob_tot + k],where a=w and k=prob_avg(1-a)+b
        # res += [w * prob_tot + k]

    return res


if __name__ == '__main__':


# coding: utf-8

# In[2]:

    import pandas as pd
    import os,sys
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    #from statsmodels.formula.api import ols
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1

    data_dir = "../../data/"
    figures_dir = "../figures/"
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks])

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # load random genes
    df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")


    # load datasets (per species)
    #datasets = ["full", "pfam", "domains", "linkers"]
    #datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

    # load datasets (per species)
    #datasets_dom = ["full", "pfam", "domains", "linkers"]
    #datasets_prot = ["full","domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]
    
    #dfs = {}
    #for ty in datasets:   
    #    df = pd.read_feather(data_dir + "df_" + ty+ "_annotation_by_species.feather")
    #    df.iupred_long = df.iupred_long*100
    #    df.iupred_short = df.iupred_short*100
    #    df.seg = df.seg*100
    #    dfs[ty] = df
        # print("TEST: ",ty,dfs[ty])


    df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species.csv")
    df=df[df['GC']  != '-']        
    #~ dfs = {}
    #~ dfs["dataset"] = pd.read_feather(data_dir + "df_annotations_by_species.feather")
    #~ dfs["dataset-eggnog"] = pd.read_feather(data_dir + "df_annotations_eggnog_by_species.feather")
    #~ dfs["dataset-pfam"] = pd.read_feather(data_dir + "df_annotations_pfam_by_species.feather")
    #~ dfs["dataset-pfam-full"] = pd.read_feather(data_dir + "df_annotations_pfam_full_sequences_by_species.feather")


    # these taxa should be excluded because their genetic code is different, so translation is wrong
    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        

    # print some statistics
    #for ds in datasets:
    #    print(ds)
    #    print(dfs[ds].kingdom.dropna().value_counts())
    #    print()
    

    # Top-IDP scale
    top_idp = {'A':0.06, 'C' :  0.02, 'D' : 0.192, 'E' : 0.736,
        'F' :  -0.697, 'G' : 0.166, 'H':0.303, 'I' :  -0.486,
        'K' : 0.586, 'L' :  -0.326, 'M': -0.397, 'N' : 0.007,
        'P' : 0.987, 'Q' : 0.318, 'R' : 0.180, 'S':  0.341,
        'T' : 0.059, 'V' :  -0.121, 'W':  -0.884, 'Y' : -0.510}

    # In[6]:

    # function to get a given AA frequency given the GC content
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
    aas_sorted_gc =     ['I','N','K','F','Y','M','L','D','C','Q','E','H','S','T','V','W','R','A','G','P']
    aas =               ['A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y']

    codons2aa = {
        'ATA':'I', 'ATC':'I', 'ATT':'I', 'ATG':'M', 'ACA':'T', 'ACC':'T', 'ACG':'T', 'ACT':'T',
        'AAC':'N', 'AAT':'N', 'AAA':'K', 'AAG':'K', 'AGC':'S', 'AGT':'S', 'AGA':'R', 'AGG':'R',
        'CTA':'L', 'CTC':'L', 'CTG':'L', 'CTT':'L', 'CCA':'P', 'CCC':'P', 'CCG':'P', 'CCT':'P',
        'CAC':'H', 'CAT':'H', 'CAA':'Q', 'CAG':'Q', 'CGA':'R', 'CGC':'R', 'CGG':'R', 'CGT':'R',
        'GTA':'V', 'GTC':'V', 'GTG':'V', 'GTT':'V', 'GCA':'A', 'GCC':'A', 'GCG':'A', 'GCT':'A',
        'GAC':'D', 'GAT':'D', 'GAA':'E', 'GAG':'E', 'GGA':'G', 'GGC':'G', 'GGG':'G', 'GGT':'G',
        'TCA':'S', 'TCC':'S', 'TCG':'S', 'TCT':'S', 'TTC':'F', 'TTT':'F', 'TTA':'L', 'TTG':'L',
        'TAC':'Y', 'TAT':'Y', 'TAA':'_', 'TAG':'_', 'TGC':'C', 'TGT':'C', 'TGA':'_', 'TGG':'W'}
        
    aa2codons = {}
    aa2codons["ALL"] = []
    for c in codons2aa:
        aa2codons[codons2aa[c]] = aa2codons.get(codons2aa[c],[]) + [c]
        if c != "_":
            aa2codons["ALL"] += [c]
            
    from operator import mul
    

    #get_aa_freq("ALL", 0.1)


    # In[7]:

    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}



    
    


    from scipy.optimize import curve_fit



    


    ##########################################################################################################################
    # for each amino acid, and separately for eukarya and bacteria, plot its frequency versus the taxonomic GC% for 4 curves:
    # 1- average freq
    # 2- a linear fit of the data
    # 3- the frequency as it should be if only dependent on GC
    # 4- a model function (w * prob_tot + k) with estimated parameters W and K
    # note: for our big dataset (from NCBI) it is also possible to plot against the GC% of the CDS

    # chose which dataset to use  for the following plots
    #df = dfs["full"]
    
    #good_taxa = set(df.taxon_id.unique())
    good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)

    res = {}

    sum_avg =  {}
    sum_gc =  {}
    sum_func = {}
    sum_func2 = {}
    sum_func3 = {}
    sum_fit =  {}
    sum_fit2 = {}

    # None of the one onepar sets work very well
    #datasets=['gc','avg','poly','lin','onepar','gcplus','gcmult','twopar']
    datasets=['gc','avg','lin','twopar','twopar-lim','gcplus','gcmult']

    
    
    
    tempcols=[]
    for setA in datasets:
        tempcols+=['err['+setA+']']
        tempcols+=['errZ['+setA+']']
        tempcols+=['abserr['+setA+']']
        tempcols+=['abssd['+setA+']']
        tempcols+=['par1['+setA+']']
        tempcols+=['par2['+setA+']']
        tempcols+=['sd1['+setA+']']
        tempcols+=['sd2['+setA+']']
        for setB in datasets:
            tempcols+=['Z['+setA+"-"+setB+']','P['+setA+"-"+setB+']']

    features=['gc']
    features+=tempcols
    cols=['kingdom', 'aa', 'gc','TOP-IDP']
    cols+=list(tempcols)


    sumlist=[]
    

    #print(cols)
    #sys.exit()
    for kingdom in ["Bacteria", "Eukaryota", "Archaea" , "Viruses"]:
        sum={}
        sumZ={}
        sumZval={}
        sumPval={}
        abssum={}
        abssumZ={}
        abssumZval={}
        abssumPval={}
        res[kingdom] = {}
        for setA in datasets:
            sum[setA] =0.0
            abssum[setA] =0.0
            sumZ[setA] =0.0
            for setB in datasets:
                sumPval[setA+"-"+setB]=0.0
                sumZval[setA+"-"+setB]=0.0

            
        fitdata={}
        for aa in aas_sorted_gc:
            
            global aminoacid
            aminoacid = aa
            global ydata_mean
            par1={}
            par2={}
            sd1={}
            sd2={}
            
            # get the distribution of the frequency in bacteria
            #d = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))][["freq_"+aa,"GC","length"]].sort_values("GC").dropna(subset=["GC"])


            #d = df.loc[(df.kingdom == kingdom) && (df.gc  != '-' )][["freq_"+aa,"GC","length"]].sort_values("GC").dropna(subset=["GC"])

            d = df.loc[(df.kingdom == kingdom) ][["freq_"+aa,"GC","length"]].sort_values("GC").dropna(subset=["GC"])
            d=d[d["GC"]  != '-']
            xdata=[]
            ydata=[]
            for a in d["GC"]:
                #print(a,type(a),float(a))
                xdata+=[float(a)/100]
            #temp_df = df.loc[(df.kingdom == kingdom)][list(set(["freq_"+aa,"GC","length"]))].dropna()
                
            #xdata = d["GC"] #/100.0
            ydata = d["freq_"+aa] # /d["length"]
            
            # get an estimate of the error given only the average AA frequence
            ydata_mean = ydata.mean()
            ydata_std = ydata.std()
            avg_ydata = [ydata_mean for _ in ydata]
            fitdata['avg']=avg_ydata            
            par1['avg']=0
            par2['avg']=0
            sd1['avg']=0
            sd2['avg']=0
        
            # do a curve fit to estimate the parameters (limited to 0,1 this is wrong)
            #par_bounds=[0.,1.]
            par_bounds=[[0.,-np.inf],[1.,+np.inf]]
            popt2, pcov2 = curve_fit(my_func, xdata,ydata, p0=[1.,0.],bounds=par_bounds )
            est2_w, est2_k = popt2
            optimized2_ydata = my_func(xdata, est2_w, est2_k)
            fitdata['twopar-lim']=optimized2_ydata
            print ("test",popt2,pcov2)
            par1['twopar-lim']=est2_w
            par2['twopar-lim']=est2_k
            sd1['twopar-lim']=np.sqrt(np.diag(pcov2[0]))
            sd2['twopar-lim']=np.sqrt(np.diag(pcov2[1]))
            print ("PAR:",par1['twopar-lim'],par2['twopar-lim'])
            print ("STD:",sd1['twopar-lim'],sd2['twopar-lim'])
            print ("tssdfg:",np.sqrt(np.diag(pcov2)))
