#!/usr/bin/env python3
from __future__ import print_function
import math

def get_aa_freq(aa,gc):
    codons = aa2codons[aa]
    if (float(gc) > 1.0):
        gc=float(gc)/100
    prob_tot = []
    for codon in codons:
        prob_codon = []
        for N in codon:
            prob = float(gc)/2.
            if N in {"A","T","U"}:
                prob = 0.5-(float(gc)/2.)
            prob_codon += [prob]
        #print(codon, prob_codon,functools.reduce(mul,prob_codon))
        prob_tot += [functools.reduce(mul,prob_codon)] # Use functools.reduce() if you really need it; however, 99 percent of the time an explicit for loop is more readable
    prob_tot = sum(prob_tot)
    return prob_tot



def get_gc_freq(aa):
    codons = aa2codons[aa]
    prob_tot = []
    a=0.
    g=0.
    for codon in codons:
        for N in codon:
            if N in {"A","T","U"}:
                a+=1.
            else:
                g+=1.
    gc=g/(g+a)
    return gc


def logtransform(aa,gc,freq):
    if math.isnan(float(gc)):
        gc=50.
    aafreq=get_aa_freq(aa,gc)
    logfreq=np.log(freq/aafreq)
    return (logfreq)


def fmt_scientific(x):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)

def running_mean(x, N):
    cumsum = pd.np.cumsum(pd.np.insert(x, 0, 0)) 
    return (cumsum[N:] - cumsum[:-N]) / N 

if __name__ == '__main__':

    import pandas as pd
    import os,sys
    import functools
    import matplotlib 
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import numpy as np
    from scipy.stats.stats import rankdata, tiecorrect
    from scipy.stats import stats
    #from statsmodels.formula.api import ols
    from scipy.stats.stats import linregress
    #matplotlib.mathtext.SHRINK_FACTOR = 1
    
    #%matplotlib inline
    from scipy.optimize import curve_fit
    
    data_dir = "../../data/"
    figures_dir = "../figures/"
    
    if not os.path.exists(figures_dir):
        os.makedirs(figures_dir)

# open the three reference files
    genomes_overview_file = data_dir + "overview.txt"
    genomes_euks_file = data_dir + "eukaryotes.txt"
    genomes_proks_file = data_dir + "prokaryotes.txt"
    
    df_overview = pd.read_csv(genomes_overview_file, sep="\t",low_memory="False")

    df_proks = pd.read_csv(genomes_proks_file, sep = "\t",low_memory="False")
    df_euks = pd.read_csv(genomes_euks_file, sep = "\t",low_memory="False")
    df_reference = pd.concat([df_euks, df_proks])

    taxid2gc = df_reference.set_index("TaxID").to_dict()["GC%"]


    taxid2group = df_reference[["TaxID","Group"]].set_index("TaxID").to_dict()["Group"]
    group2kingdom = df_overview[["Group", "Kingdom"]].set_index("Group").to_dict()["Kingdom"]
    taxid2kingdom = {}

    for t in taxid2group:
        gr = taxid2group[t]
        taxid2kingdom[t] = group2kingdom[gr]
    
    taxid2name = df_reference[["#Organism/Name","TaxID"]].set_index("TaxID").to_dict()['#Organism/Name']
    name2taxid = df_reference[["#Organism/Name","TaxID"]].set_index("#Organism/Name").to_dict()['TaxID']


    # load random genes
    df_genes_random = pd.read_feather(data_dir + "df_genes_random.feather")



    # load datasets (per species)
    #datasets = ["full", "pfam", "domains", "linkers"]
    #datasets += ["domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]

    # load datasets (per species)
    #datasets_dom = ["full", "pfam", "domains", "linkers"]
    #datasets_prot = ["full","domains_exclusive_euk","domains_exclusive_bac","domains_orthologs_5"]
    
    #dfs = {}
    #for ty in datasets:   
    #    df = pd.read_feather(data_dir + "df_" + ty+ "_annotation_by_species.feather")
    #    df.iupred_long = df.iupred_long*100
    #    df.iupred_short = df.iupred_short*100
    #    df.seg = df.seg*100
    #    dfs[ty] = df
        # print("TEST: ",ty,dfs[ty])


    df = pd.read_csv(data_dir + "df_uniprot_reference_proteomes_per_species.csv")
    df=df[df['GC']  != '-']        
    #~ dfs = {}
    #~ dfs["dataset"] = pd.read_feather(data_dir + "df_annotations_by_species.feather")
    #~ dfs["dataset-eggnog"] = pd.read_feather(data_dir + "df_annotations_eggnog_by_species.feather")
    #~ dfs["dataset-pfam"] = pd.read_feather(data_dir + "df_annotations_pfam_by_species.feather")
    #~ dfs["dataset-pfam-full"] = pd.read_feather(data_dir + "df_annotations_pfam_full_sequences_by_species.feather")


    # these taxa should be excluded because their genetic code is different, so translation is wrong
    # these taxa should be excluded because their genetic code is different, so translation is wrong
    mycotaxa = []
    for t in name2taxid:
        tax = t.split()[0]
        if tax in ["Mycoplasma","Spiroplasma","Ureaplasma","Mesoplasma"]:
            mycotaxa += [name2taxid[t]]
        
    dic_labels = {"GC":"GC%","GC_genomic":"GC% (genomic)",                'other':'Intermediate',                'ancient':'Ancient','orphan':'Orphans','group_orphan':"Genus\norphans",
    'hessa':"<Hydrophobicity> (Hessa)",                "top-idp":"TOP-IDP", "ss_alpha":"<Alpha propensity>","ss_beta":"<Beta propensity>","ss_coil":"<Coil propensity>",
    'ss_turn':"<Turn propensity>","length":"Length (AA)",                'gc_third':'GC% (third nucleotide)','taxon_gc':'GC%        (taxonomic avg)','taxon_gc_r':'GC% (taxonomic avg)',
                'random':'Expected','iupred_long':'IUpred long(%AA)','length_aa':'<Length> (AA)','gc':'GC%',
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff (%AA)',
                'idp':'<TOP-IDP>','tm':'TM (%AA)','transmembrane':'TM(%AA)',"volatility":"<Volatility>",
                'iupred_short':'IUpred short(%AA)','iupred_cutoff':'IUpred cutoff(%AA)','seg':'<SEG>', 'hydropathy':'<Hydrophobicity>(Kyte-Doolittle)','gc':'GC%','aa_corr':'AA freq\ncorrelation with expected',
                 'aa_corr_dot':'AA freq\ncorrelation with expected','aa_corr_r':'AA freq\ncorrelation with expected', 'A': 'ALA', 'C':
                'CYS', 'E': 'GLU', 'D': 'ASP', 'G': 'GLY', 'F': 'PHE',
                'I': 'ILE', 'H': 'HIS', 'K': 'LYS', 'M': 'MET', 'L':
                'LEU', 'N': 'ASN', 'Q': 'GLN', 'P': 'PRO', 'S': 'SER',
                'R': 'ARG', 'T': 'THR', 'W': 'TRP', 'V': 'VAL', 'Y':
                'TYR', 'freq_A': 'ALA', 'freq_C': 'CYS', 'freq_E': 'GLU',
                      'freq_D': 'ASP', 'freq_G': 'GLY', 'freq_F': 'PHE',
                      'freq_I': 'ILE', 'freq_H': 'HIS', 'freq_K': 'LYS',
                      'freq_M': 'MET', 'freq_L': 'LEU', 'freq_N': 'ASN',
                      'freq_Q': 'GLN', 'freq_P': 'PRO', 'freq_S':'SER', 'freq_R': 'ARG',
                      'freq_T': 'THR', 'freq_W':                'TRP', 'freq_V': 'VAL',
                      'freq_Y': 'TYR', 'scampi':'TM  (%AA)', 'slider':'Slider',
                      'disembl_coils':'DisEMBL                Coils (%AA)',
                      'disembl_rem465':"DisEMBL REM465 (%AA)",
                      'disembl_hotloops':"DisEMBL Hotloops (%AA)",
                      "globpipe":"GlobPlot (%AA)", "euk":"Eukaryota",
                "pro":"Bacteria",
                "arc":"Archaea",'Eukaryota':'Eukaryota','Bacteria':'Bacteria','Archaea':'Archaea','Viruses':'Viruses'}



    # function to get a given AA frequency given the GC content
    aas_sorted_topidp = ['W','F','Y','I','M','L','V','N','C','T','A','G','R','D','H','Q','S','K','E','P']
    aas_sorted_gc =     ['I','N','K','F','Y','M','L','D','C','Q','E','H','S','T','V','W','R','A','G','P']
    aas =               ['A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y']

    codons2aa = {
        'ATA':'I', 'ATC':'I', 'ATT':'I', 'ATG':'M', 'ACA':'T', 'ACC':'T', 'ACG':'T', 'ACT':'T',
        'AAC':'N', 'AAT':'N', 'AAA':'K', 'AAG':'K', 'AGC':'S', 'AGT':'S', 'AGA':'R', 'AGG':'R',
        'CTA':'L', 'CTC':'L', 'CTG':'L', 'CTT':'L', 'CCA':'P', 'CCC':'P', 'CCG':'P', 'CCT':'P',
        'CAC':'H', 'CAT':'H', 'CAA':'Q', 'CAG':'Q', 'CGA':'R', 'CGC':'R', 'CGG':'R', 'CGT':'R',
        'GTA':'V', 'GTC':'V', 'GTG':'V', 'GTT':'V', 'GCA':'A', 'GCC':'A', 'GCG':'A', 'GCT':'A',
        'GAC':'D', 'GAT':'D', 'GAA':'E', 'GAG':'E', 'GGA':'G', 'GGC':'G', 'GGG':'G', 'GGT':'G',
        'TCA':'S', 'TCC':'S', 'TCG':'S', 'TCT':'S', 'TTC':'F', 'TTT':'F', 'TTA':'L', 'TTG':'L',
        'TAC':'Y', 'TAT':'Y', 'TAA':'_', 'TAG':'_', 'TGC':'C', 'TGT':'C', 'TGA':'_', 'TGG':'W'}
        
    aa2codons = {}
    aa2codons["ALL"] = []
    for c in codons2aa:
        aa2codons[codons2aa[c]] = aa2codons.get(codons2aa[c],[]) + [c]
        if c != "_":
            aa2codons["ALL"] += [c]
            
    from operator import mul
    
    

#sys.exit(1)
    ######################################################

    # for a given dataset (ex. "dataset") plot two properties against each other

    #df = dfs["full"]

    # the properties to plot
    properties_x = ["GC"]
    #properties_x = ["GC", "GC3", "GC_genomic"]

    #~ properties_y = [u'freq_A', u'freq_C', u'freq_D']
    properties_y = [u'top-idp', u'hessa', u'iupred_long', u'iupred_short', u'seg',
                    'ss_alpha', u'ss_beta', u'ss_coil', u'ss_turn',
        u'log_A', u'log_C', u'log_D', u'log_E', u'log_F', u'log_G',
        u'log_H', u'log_I', u'log_K', u'log_L', u'log_M', u'log_N',
        u'log_P', u'log_Q', u'log_R', u'log_S', u'log_T', u'log_V',
        u'log_W', u'log_Y']
    properties_y = [u'top-idp', u'iupred_long', u'iupred_short',
        u'log_A', u'log_C', u'log_D', u'log_E', u'log_F', u'log_G',
        u'log_H', u'log_I', u'log_K', u'log_L', u'log_M', u'log_N',
        u'log_P', u'log_Q', u'log_R', u'log_S', u'log_T', u'log_V',
        u'log_W', u'log_Y']
        
        
        
        
    dic_limits={"hessa":[0.85,1.25],"GC":[20,80],"GC_genomic":[20,80],"taxon_gc":[30,65],"taxon_gc_r":[30,65],"gc_third":[30,65],
                'aa_corr':[0.8,1],'aa_corr_dot':[0.8,1.0],'aa_corr_r':[0.4,1],
                "top-idp":[0,0.15], 'W':[0.006,0.02], 'C':[0.01,0.04],
                'H':[0.01,0.04], 'M':[0.01,0.04], 'Y':[0.01,0.06],
                'Q':[0.02,0.05], 'P':[0.02,0.11], 'F':[0.01,0.08],
                'R':[0.02,0.1], 'G':[0,0.12], 'D':[0.02,0.06],
                'A':[0,0.15], 'V':[0.04,0.08], 'N':[0.01,0.08],
                'T':[0.04,0.08], 'E':[0.02,0.08], 'I':[0,0.12],
                'K':[0.01,0.1], 'S':[0.05,0.11], 'L':[0.06,0.12],
                'freq_W':[0.0,0.025], 'freq_C':[0.00,0.04],
                'freq_H':[0.01,0.035], 'freq_M':[0.01,0.065],
                'freq_Y':[0.01,0.06], 'freq_Q':[0.01,0.07],
                'freq_P':[0.0,0.08], 'freq_F':[0.018,0.08],
                'freq_R':[0.02,0.11], 'freq_G':[0.02,0.12],
                'freq_D':[0.02,0.1], 'freq_A':[0,0.18],
                'freq_V':[0.04,0.11], 'freq_N':[0.01,0.095],
                'freq_T':[0.03,0.08], 'freq_E':[0.02,0.1],
                'freq_I':[0.02,0.14], 'freq_K':[0.01,0.13],
                'freq_S':[0.04,0.11], 'freq_L':[0.075,0.15],
                "iupred_long":[0,40], "tm":[0,12],'idp':[0,0.20],
                "seg":[0,25],"length_aa":[0,720],'iupred_short':[0,40],
                'iupred_cutoff':[0,50],'volatility':[0.7,0.85],'hydropathy':[-1,1],
                "ss_alpha":[-0.05,0.03],"ss_beta":[-0.1,0.02],"ss_coil":[-0.03,0.02],"ss_turn":[-0.11,0.0],
                'slider':[10,80],'disembl_coils':[40,100],'disembl_rem465':[0,80],'disembl_hotloops':[10,90],
                "globpipe":[0,100] } # hardcoded axis limits


    dic_colors = {"Eukaryota":"#9daf6a", "Bacteria":"#b45a5c", "Archaea":"#377eb8"}
    dic_colors_fit = {"Eukaryota":"#bdcf8a", "Bacteria":"#f43a3c", "Archaea":"#579ed8"}
    dic_colors_mean = {"Eukaryota":"#9daf6a", "Bacteria":"#b45a5c", "Archaea":"#377eb8"}
    dic_markers = {"Eukaryota":"o", "Bacteria":"x", "Archaea":"o"}
        

    prop_1_values = {}
    prop_2_values = {}

    
    # First we need to define a function that calculates expected AA freq give GC
    for aa in aas_sorted_topidp:
        df['log_'+aa]=np.vectorize(logtransform)(aa, df['GC'],df['freq_'+aa])

    print (df)
        
    #for i in aa:
    #    df.apply (lambda row: label_race (row),axis=1)
    
    for prop_1 in properties_x:
        for prop_2 in properties_y:

            fig, ax = plt.subplots(figsize=(6,6))
        
            err_mean={}
            err_fit={}
            # plot the values as a scatter plot
            for kingdom in ['Eukaryota','Bacteria','Archaea']:
                prop_1_values[kingdom] = []
                prop_2_values[kingdom] = []


                good_taxa = set(df.taxon_id.unique()) - set(mycotaxa)
                
                #temp_df = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa))][list(set([prop_1,prop_2,"iupred_short","length"]))].dropna()
                #temp_df=df[df[prop_1]  != '-']        
                
                #prop_1_values[kingdom][kingdom] = temp_df[prop_1].values
                
                #prop_2_values[kingdom] = temp_df[prop_2]/temp_df["length"]
                
                temp_df = df.loc[(df.kingdom == kingdom) & (df.taxon_id.isin(good_taxa) )][list(set([prop_1,prop_2]))].dropna()
                temp_df=temp_df[temp_df[prop_1]  != '-']
                #temp_df[df[prop_2].apply(lambda x: x.isnumeric())]

                #print("TEST",prop_2,kingdom,temp_df.shape)
                prop_1_values[kingdom] = []
                for a in temp_df[prop_1].values:
                #print(a,type(a),float(a))
                    prop_1_values[kingdom]+= [float(a)]
                prop_2_values[kingdom] = temp_df[prop_2].values

                ax.scatter(prop_1_values[kingdom],prop_2_values[kingdom], edgecolor="none", facecolor=dic_colors[kingdom],alpha = 1,s=50,label=dic_labels[kingdom])
                #sct = ax.scatter(prop_1_values[kingdom],prop_2_values[kingdom],marker=dic_markers[kingdom], c=colors_list, edgecolor="none",alpha = 1,s=70,label=dic_labels[kingdom])
                
                # calculate and plot the fitting lines
                
                print(kingdom,prop_1,prop_2)

                #print(prop_1_values[kingdom])
                #print(prop_2_values[kingdom])
                
                fit = np.polyfit(list(prop_1_values[kingdom]),list(prop_2_values[kingdom]),1)
                fit_fn = np.poly1d(fit) 
                plt.plot(prop_1_values[kingdom], fit_fn(prop_1_values[kingdom]), '--k',lw=2,color=dic_colors_fit[kingdom])
                n=len(prop_1_values[kingdom])
                fit_values = np.polyval(fit, prop_1_values[kingdom])
                err_fit[kingdom] = np.sqrt(sum((fit_values - prop_2_values[kingdom])**2)/n)

                mean = np.polyfit(list(prop_1_values[kingdom]),list(prop_2_values[kingdom]),0)
                mean_fn = np.poly1d(mean) 
                plt.plot(prop_1_values[kingdom], mean_fn(prop_1_values[kingdom]), '.',lw=1,color=dic_colors_fit[kingdom])
                mean_values = np.polyval(mean, prop_1_values[kingdom])
                err_mean[kingdom] = np.sqrt(sum((mean_values - prop_2_values[kingdom])**2)/n)
                
                slope, intercept, r_value, p_value, std_err = linregress(prop_1_values[kingdom], prop_2_values[kingdom])
                print("LinReg:",prop_1,prop_2,kingdom,slope, intercept, r_value, p_value, std_err)
                print ("Errors",err_fit,err_mean)

            # plot the random running average
            if prop_1 in df_genes_random.columns and prop_2 in df_genes_random.columns:
                prop_1_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_1].astype(float)),500)
                prop_2_random = running_mean(pd.np.array(df_genes_random.sort_values(prop_1)[prop_2].astype(float)),500)
                #ax.plot(prop_1_random,prop_2_random,color="black",label="Random")
                
            # plot the theoretical line
            if prop_2.split("_")[0] == "log" and prop_1 in ["GC","GC3","GC_genomic"]:
                aa = prop_2.split("_")[1]
                p = []
                for gc in np.arange(0.1,0.9,0.001):
                    p += [get_aa_freq(aa, gc)]
                
                ax.plot(np.arange(10,90,0.1),p,color="blue",lw=2,label="Theoretical")


            # set the limits
            ax.set_xlim(-1.5,1.5)
            if prop_1 in dic_limits:
                ax.set_xlim(dic_limits[prop_1])
                
            ax.set_ylim(-1.5,1.5)
            if prop_2 in dic_limits:
                ax.set_ylim(dic_limits[prop_2])
                
                
            # set the labels
            ax.set_xlabel(dic_labels.get(prop_1,prop_1))
            ax.set_ylabel(dic_labels.get(prop_2,prop_2))
            
            # plot legend
            ax.legend(loc="upper right")
            
            # ranksums tests
            ranksum_string = ''
            #ranksum_string += "Eukarya vs Bacteria = " + fmt_scientific(stats.ranksums(prop_2_values["Eukaryota"], prop_2_values["Bacteria"])[1]) + "\n"
            #ranksum_string += "Eukarya vs Archaea = " + fmt_scientific(stats.ranksums(prop_2_values["Eukaryota"], prop_2_values["Archaea"])[1]) + "\n"
            #ranksum_string += "Bacteria vs Archaea = " + fmt_scientific(stats.ranksums(prop_2_values["Bacteria"], prop_2_values["Archaea"])[1]) + "\n" + "\n"
            
            # Pearson correlation coefficient
            #ranksum_string += "Eukarya R = " + str(stats.pearsonr(prop_1_values["Eukaryota"], prop_2_values["Eukaryota"])[0]) + "\n"
            #ranksum_string += "Bacteria R = " + str(stats.pearsonr(prop_1_values["Bacteria"], prop_2_values["Bacteria"])[0]) + "\n"
            #ranksum_string += "Archaea R = " + str(stats.pearsonr(prop_1_values["Archaea"], prop_2_values["Archaea"])[0])
            
            
            # calculate ABS difference between Euk-theoretical and Prok-theoretical
            if prop_2.split("_")[0] == "log" and prop_1 in ["GC","GC3","GC_genomic"]:
                
                diff_from_theo = {}
                
                aa = prop_2.split("_")[1]
                for kingdom in ['Eukaryota','Bacteria','Archaea']:
                    p = []
                    ranksum_string += kingdom + "-Error Mean/Lin "+str(round(err_mean[kingdom]/err_fit[kingdom],4))+"\n"
                for kingdom in ['Eukaryota','Bacteria','Archaea']:
                    p = []
                    for vx,vy in zip (prop_1_values[kingdom], prop_2_values[kingdom]):
                        p += [vy-get_aa_freq(aa, vx/100.0)]
                        
                    ranksum_string +=   kingdom + " - theoretical = " + str(round(np.mean(p),4))+ "\n"
                    
                    
            bb = ax.text(ax.get_xlim()[0], ax.get_ylim()[1], ranksum_string, fontsize=6,
                ha="left", va="top",
                bbox=dict(boxstyle="square,pad=0",
                        ec=(0.0, 0.0, 0.0,1),
                        fc=(1., 1, 1,1),),    
                    )
            
        
            
            figname = figures_dir + prop_2 + "_vs_" + prop_1 + "_uniprot.eps"
            print(figname)
            fig.savefig(figname)
            plt.close()
            #fig.show()
            
            ##########################################################################################################################
